"""A function for decrypting an odds portal dat file."""

# pylint: disable=too-many-locals
import base64
import json
import logging
import urllib.parse
from typing import Any

import requests
import requests_cache
from bs4 import BeautifulSoup
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.hashes import SHA256
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from ...proxy_session import X_NO_WAYBACK


def _find_decryption_data(
    session: requests_cache.CachedSession,
    soup: BeautifulSoup,
    referer_url: str,
    user_agent: str | None = None,
) -> tuple[bytes, bytes]:
    salt: bytes | None = None
    password: bytes | None = None
    for script in soup.find_all("script"):
        src = script.get("src")
        if src is None:
            continue
        if ("/app-" in src and src.endswith(".js")) or "/app.js" in src:
            src_url = urllib.parse.urljoin(referer_url, src)
            headers = {X_NO_WAYBACK: "1"}
            if user_agent is not None:
                headers["User-Agent"] = user_agent
            src_response = session.get(src_url, headers=headers)
            src_response.raise_for_status()
            variables = src_response.text
            sentinel = 'break}return e.next=9,g(r.data,"'
            variables = variables[variables.find(sentinel) + len(sentinel) :]
            variables = variables[
                : variables.find('");case 9:return s=e.sent,l=JSON.parse(s),e.abrupt')
            ]
            try:
                password_str, salt_str = variables.split('","')
            except ValueError:
                sentinel = "YupiOddsPortal"
                variables = src_response.text
                variables = variables[: variables.find(sentinel) + len(sentinel)]
                variables = '"'.join(variables.split('"')[-3:])
                password_str, salt_str = variables.split('","')
            salt = str.encode(salt_str)
            password = str.encode(password_str)
            break
    if salt is None:
        raise ValueError(f"salt is null for {referer_url}.")
    if password is None:
        raise ValueError("password is null.")
    return salt, password


def fetch_data(
    url: str,
    session: requests_cache.CachedSession,
    referer_url: str,
    soup: BeautifulSoup,
    user_agent: str | None = None,
) -> dict[str, Any]:
    """Fetch the data from the URL and decrypt it."""
    salt, password = _find_decryption_data(
        session, soup, referer_url, user_agent=user_agent
    )
    headers = {
        "X-Requested-With": "XMLHttpRequest",
        "Referer": referer_url,
    }
    if user_agent is not None:
        headers["User-Agent"] = user_agent
    response = session.get(
        url,
        headers=headers,
    )

    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError as exc:
        logging.error("HTTP error: %s for URL: %s", str(exc), referer_url)
        raise exc

    try:
        decoded_data = base64.b64decode(response.content).decode()
    except UnicodeDecodeError as exc:
        logging.error("URL: %s", url)
        logging.error("Error base64 decoding payload: %s", response.content)
        raise exc
    encrypted, key = decoded_data.split(":")
    encrypted_bytes = base64.urlsafe_b64decode(encrypted)
    key_bytes = bytes.fromhex(key)
    kdf = PBKDF2HMAC(
        algorithm=SHA256(),
        length=32,
        salt=salt,
        iterations=1000,
        backend=default_backend(),
    )
    aes_key = kdf.derive(password)
    cipher = Cipher(
        algorithms.AES(aes_key), modes.CBC(key_bytes), backend=default_backend()
    )
    decryptor = cipher.decryptor()
    decrypted_bytes = decryptor.update(encrypted_bytes) + decryptor.finalize()
    decrypted_data = decrypted_bytes.decode("utf-8")
    end_of_json = decrypted_data.rfind("}")
    if end_of_json != -1:
        decrypted_data = decrypted_data[: end_of_json + 1]
    return json.loads(decrypted_data)



"""AFL AFLTables team model."""

# pylint: disable=too-many-arguments,too-many-locals,duplicate-code,line-too-long
import datetime
import os
import urllib.parse
from urllib.parse import urlparse

import pytest_is_running
import requests_cache
from bs4 import BeautifulSoup

from ....cache import MEMORY
from ...google.google_news_model import create_google_news_models
from ...league import League
from ...team_model import TeamModel
from ...x.x_social_model import create_x_social_model
from .afl_afltables_coach_model import create_afl_afltables_coach_model
from .afl_afltables_player_model import create_afl_afltables_player_model

_TEAM_NAME_MAP = {
    "Melbourne": ["ME"],
    "Geelong": ["GE"],
    "Fitzroy": ["FI"],
    "Collingwood": ["CW"],
    "Essendon": ["ES"],
    "South Melbourne": ["SM"],
    "St Kilda": ["SK"],
    "Carlton": ["CA"],
    "Sydney": ["SM", "SY"],
    "University": ["UN"],
    "Richmond": ["RI"],
    "North Melbourne": ["NM"],
    "Western Bulldogs": ["WB", "FO"],
    "Hawthorn": ["HW"],
    "Brisbane Bears": ["BB"],
    "West Coast": ["WC"],
    "Adelaide": ["AD"],
    "Fremantle": ["FR"],
    "Brisbane Lions": ["BL"],
    "Port Adelaide": ["PA"],
    "Gold Coast": ["GC"],
    "Greater Western Sydney": ["GW"],
}


def _create_afl_afltables_team_model(
    team_url: str,
    players: list[
        tuple[
            str,
            str,
            int | None,
            str,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            float | None,
        ]
    ],
    points: float,
    session: requests_cache.CachedSession,
    last_ladder_ranks: dict[str, int] | None,
    dt: datetime.datetime,
    league: League,
) -> TeamModel:
    response = session.get(team_url)
    soup = BeautifulSoup(response.text, "lxml")
    o = urlparse(team_url)
    last_component = o.path.split("/")[-1]
    identifier, _ = os.path.splitext(last_component)
    h1 = soup.find("h1")
    if h1 is None:
        raise ValueError("h1 is null.")
    name = h1.get_text()
    last_ladder_rank = None
    if last_ladder_ranks is not None and last_ladder_ranks:
        short_names = _TEAM_NAME_MAP[name]
        for short_name in short_names:
            if short_name in last_ladder_ranks:
                last_ladder_rank = last_ladder_ranks[short_name]
                break
    coaches_url = None
    for a in soup.find_all("a", href=True):
        a_text = a.get_text()
        if a_text == "Coaches":
            coaches_url = urllib.parse.urljoin(team_url, a.get("href"))
            break
    if coaches_url is None:
        raise ValueError("coaches_url is null")

    coach_model = create_afl_afltables_coach_model(
        url=coaches_url, session=session, year=dt.year, dt=dt
    )

    return TeamModel(
        identifier=identifier,
        name=name,
        players=[  # pyright: ignore
            create_afl_afltables_player_model(
                player_url=player_url,
                jersey=jersey,
                kicks=kicks,
                session=session,
                name=name,
                marks=marks,
                handballs=handballs,
                disposals=disposals,
                goals=goals,
                behinds=behinds,
                hit_outs=hit_outs,
                tackles=tackles,
                rebounds=rebounds,
                insides=insides,
                clearances=clearances,
                clangers=clangers,
                free_kicks_for=free_kicks_for,
                free_kicks_against=free_kicks_against,
                brownlow_votes=brownlow_votes,
                contested_possessions=contested_possessions,
                uncontested_possessions=uncontested_possessions,
                contested_marks=contested_marks,
                marks_inside=marks_inside,
                one_percenters=one_percenters,
                bounces=bounces,
                goal_assists=goal_assists,
                percentage_played=percentage_played,
                dt=dt,
            )
            for player_url, jersey, kicks, name, marks, handballs, disposals, goals, behinds, hit_outs, tackles, rebounds, insides, clearances, clangers, free_kicks_for, free_kicks_against, brownlow_votes, contested_possessions, uncontested_possessions, contested_marks, marks_inside, one_percenters, bounces, goal_assists, percentage_played in players
        ],
        odds=[],
        points=points,
        ladder_rank=last_ladder_rank,
        location=None,
        news=create_google_news_models(name, session, dt, league),
        social=create_x_social_model(identifier, session, dt),
        field_goals=None,
        coaches=[coach_model] if coach_model is not None else [],
        lbw=None,
        end_dt=None,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_afl_afltables_team_model(
    team_url: str,
    players: list[
        tuple[
            str,
            str,
            int | None,
            str,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            float | None,
        ]
    ],
    points: float,
    session: requests_cache.CachedSession,
    last_ladder_ranks: dict[str, int] | None,
    dt: datetime.datetime,
    league: League,
) -> TeamModel:
    return _create_afl_afltables_team_model(
        team_url,
        players,
        points,
        session,
        last_ladder_ranks,
        dt,
        league,
    )


def create_afl_afltables_team_model(
    team_url: str,
    players: list[
        tuple[
            str,
            str,
            int | None,
            str,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            int | None,
            float | None,
        ]
    ],
    points: float,
    session: requests_cache.CachedSession,
    last_ladder_ranks: dict[str, int] | None,
    dt: datetime.datetime,
    league: League,
) -> TeamModel:
    """Create a team model from AFL Tables."""
    if not pytest_is_running.is_running():
        return _cached_create_afl_afltables_team_model(
            team_url, players, points, session, last_ladder_ranks, dt, league
        )
    with session.cache_disabled():
        return _create_afl_afltables_team_model(
            team_url, players, points, session, last_ladder_ranks, dt, league
        )



"""AFL AFL team model."""

# pylint: disable=duplicate-code,too-many-arguments
import datetime

import requests_cache

from ...google.google_news_model import create_google_news_models
from ...league import League
from ...team_model import TeamModel
from ..position import Position
from .afl_afl_odds_model import create_afl_afl_odds_model
from .afl_afl_player_model import create_afl_afl_player_model


def create_afl_afl_team_model(
    team_name: str,
    players: list[tuple[str, str, str, str, Position]],
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    ladder: list[str],
    odds: float | None,
) -> TeamModel:
    """Create a team model from AFL AFL."""
    player_models = [
        create_afl_afl_player_model(
            identifier, player_number, " ".join([first_name, second_name]), position
        )
        for identifier, player_number, first_name, second_name, position in players
    ]
    odds_models = []
    if odds is not None:
        odds_models.append(create_afl_afl_odds_model(odds))
    if team_name not in ladder:
        raise ValueError(f"{team_name} not found in ladder")
    return TeamModel(
        identifier=team_name,
        name=team_name,
        location=None,
        players=player_models,
        odds=odds_models,
        points=None,
        ladder_rank=ladder.index(team_name) + 1,
        kicks=None,
        news=create_google_news_models(team_name, session, dt, League.AFL),
        social=[],
        field_goals=None,
        field_goals_attempted=None,
        offensive_rebounds=None,
        assists=None,
        turnovers=None,
        coaches=[],
        lbw=None,
        end_dt=None,
    )



"""Tests for the team model class."""
import unittest

from sportsball.data.team_model import TeamModel


class TestTeamModel(unittest.TestCase):

    def setUp(self):
        self._team_model = TeamModel(
            identifier="A",
            name="B",
            location=None,
            players=[],
            odds=[],
            points=2.0,
            ladder_rank=10,
            kicks=None,
            news=[],
            social=[],
            field_goals=None,
            coaches=[],
            lbw=None,
            end_dt=None,
        )

    def test_notnull(self):
        self.assertIsNotNone(self._team_model)



"""NCAAF combined league model."""

import requests_cache

from ...combined.combined_league_model import CombinedLeagueModel
from ...league import League
from ..espn.ncaaf_espn_league_model import NCAAFESPNLeagueModel
from ..oddsportal.ncaaf_oddsportal_league_model import \
    NCAAFOddsPortalLeagueModel

NCAAF_TEAM_IDENTITY_MAP: dict[str, str] = {}
NCAAF_VENUE_IDENTITY_MAP: dict[str, str] = {}
NCAAF_PLAYER_IDENTITY_MAP: dict[str, str] = {}


class NCAAFCombinedLeagueModel(CombinedLeagueModel):
    """NCAAF combined implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, league_filter: str | None
    ) -> None:
        super().__init__(
            session,
            League.NCAAF,
            [
                NCAAFESPNLeagueModel(session, position=0),
                NCAAFOddsPortalLeagueModel(session, position=1),
            ],
            league_filter,
        )

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        return NCAAF_TEAM_IDENTITY_MAP

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        return NCAAF_VENUE_IDENTITY_MAP

    @classmethod
    def player_identity_map(cls) -> dict[str, str]:
        return NCAAF_PLAYER_IDENTITY_MAP

    @classmethod
    def name(cls) -> str:
        return "ncaaf-combined-league-model"



"""AFL AFLTables player model."""

# pylint: disable=line-too-long,duplicate-code,too-many-arguments,too-many-locals
import datetime
import logging
import os
from urllib.parse import urlparse

import pytest_is_running
import requests_cache
from bs4 import BeautifulSoup
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta

from ....cache import MEMORY
from ...player_model import PlayerModel
from ...sex import Sex
from ...species import Species


def _create_afl_afltables_player_model(
    player_url: str,
    jersey: str,
    kicks: int | None,
    name: str,
    marks: int | None,
    handballs: int | None,
    disposals: int | None,
    goals: int | None,
    behinds: int | None,
    hit_outs: int | None,
    tackles: int | None,
    rebounds: int | None,
    insides: int | None,
    clearances: int | None,
    clangers: int | None,
    free_kicks_for: int | None,
    free_kicks_against: int | None,
    brownlow_votes: int | None,
    contested_possessions: int | None,
    uncontested_possessions: int | None,
    contested_marks: int | None,
    marks_inside: int | None,
    one_percenters: int | None,
    bounces: int | None,
    goal_assists: int | None,
    percentage_played: float | None,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
) -> PlayerModel:
    o = urlparse(player_url)
    last_component = o.path.split("/")[-1]
    identifier, _ = os.path.splitext(last_component)
    jersey = "".join(filter(str.isdigit, jersey))
    response = session.get(player_url)
    response.raise_for_status()

    soup = BeautifulSoup(response.text, "lxml")
    player_page_text = soup.get_text()

    birth_date = None
    try:
        birth_date = parse(
            player_page_text.split("Born:")[1].strip().split()[0].strip()
        )
    except IndexError:
        logging.warning("Couldn't find birth date from %s", response.url)

    weight = None
    try:
        weight = float(player_page_text.split("Weight:")[1].strip().split()[0].strip())
    except IndexError:
        logging.warning("Couldn't find weight from %s", response.url)

    return PlayerModel(
        identifier=identifier,
        jersey=jersey,
        kicks=kicks,
        fumbles=None,
        fumbles_lost=None,
        field_goals=None,
        field_goals_attempted=None,
        offensive_rebounds=None,
        assists=None,
        turnovers=None,
        name=name,
        marks=marks,
        handballs=handballs,
        disposals=disposals,
        goals=goals,
        behinds=behinds,
        hit_outs=hit_outs,
        tackles=tackles,
        rebounds=rebounds,
        insides=insides,
        clearances=clearances,
        clangers=clangers,
        free_kicks_for=free_kicks_for,
        free_kicks_against=free_kicks_against,
        brownlow_votes=brownlow_votes,
        contested_possessions=contested_possessions,
        uncontested_possessions=uncontested_possessions,
        contested_marks=contested_marks,
        marks_inside=marks_inside,
        one_percenters=one_percenters,
        bounces=bounces,
        goal_assists=goal_assists,
        percentage_played=percentage_played,
        birth_date=birth_date,
        species=str(Species.HUMAN),
        handicap_weight=None,
        father=None,
        sex=str(Sex.MALE),
        age=None if birth_date is None else relativedelta(birth_date, dt).years,
        starting_position=None,
        weight=weight,
        birth_address=None,
        owner=None,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_afl_afltables_player_model(
    player_url: str,
    jersey: str,
    kicks: int | None,
    name: str,
    marks: int | None,
    handballs: int | None,
    disposals: int | None,
    goals: int | None,
    behinds: int | None,
    hit_outs: int | None,
    tackles: int | None,
    rebounds: int | None,
    insides: int | None,
    clearances: int | None,
    clangers: int | None,
    free_kicks_for: int | None,
    free_kicks_against: int | None,
    brownlow_votes: int | None,
    contested_possessions: int | None,
    uncontested_possessions: int | None,
    contested_marks: int | None,
    marks_inside: int | None,
    one_percenters: int | None,
    bounces: int | None,
    goal_assists: int | None,
    percentage_played: float | None,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
) -> PlayerModel:
    return _create_afl_afltables_player_model(
        player_url=player_url,
        jersey=jersey,
        kicks=kicks,
        name=name,
        marks=marks,
        handballs=handballs,
        disposals=disposals,
        goals=goals,
        behinds=behinds,
        hit_outs=hit_outs,
        tackles=tackles,
        rebounds=rebounds,
        insides=insides,
        clearances=clearances,
        clangers=clangers,
        free_kicks_for=free_kicks_for,
        free_kicks_against=free_kicks_against,
        brownlow_votes=brownlow_votes,
        contested_possessions=contested_possessions,
        uncontested_possessions=uncontested_possessions,
        contested_marks=contested_marks,
        marks_inside=marks_inside,
        one_percenters=one_percenters,
        bounces=bounces,
        goal_assists=goal_assists,
        percentage_played=percentage_played,
        session=session,
        dt=dt,
    )


def create_afl_afltables_player_model(
    player_url: str,
    jersey: str,
    kicks: int | None,
    session: requests_cache.CachedSession,
    name: str,
    marks: int | None,
    handballs: int | None,
    disposals: int | None,
    goals: int | None,
    behinds: int | None,
    hit_outs: int | None,
    tackles: int | None,
    rebounds: int | None,
    insides: int | None,
    clearances: int | None,
    clangers: int | None,
    free_kicks_for: int | None,
    free_kicks_against: int | None,
    brownlow_votes: int | None,
    contested_possessions: int | None,
    uncontested_possessions: int | None,
    contested_marks: int | None,
    marks_inside: int | None,
    one_percenters: int | None,
    bounces: int | None,
    goal_assists: int | None,
    percentage_played: float | None,
    dt: datetime.datetime,
) -> PlayerModel:
    """Create a player model from AFL Tables."""
    if not pytest_is_running.is_running():
        return _cached_create_afl_afltables_player_model(
            player_url=player_url,
            jersey=jersey,
            kicks=kicks,
            name=name,
            marks=marks,
            handballs=handballs,  # pyright: ignore
            disposals=disposals,
            goals=goals,
            behinds=behinds,
            hit_outs=hit_outs,
            tackles=tackles,
            rebounds=rebounds,
            insides=insides,
            clearances=clearances,
            clangers=clangers,
            free_kicks_for=free_kicks_for,
            free_kicks_against=free_kicks_against,
            brownlow_votes=brownlow_votes,
            contested_possessions=contested_possessions,
            uncontested_possessions=uncontested_possessions,
            contested_marks=contested_marks,
            marks_inside=marks_inside,
            one_percenters=one_percenters,
            bounces=bounces,
            goal_assists=goal_assists,
            percentage_played=percentage_played,
            session=session,
            dt=dt,
        )
    with session.cache_disabled():
        return _create_afl_afltables_player_model(
            player_url=player_url,
            jersey=jersey,
            kicks=kicks,
            name=name,
            marks=marks,
            handballs=handballs,
            disposals=disposals,
            goals=goals,
            behinds=behinds,
            hit_outs=hit_outs,
            tackles=tackles,
            rebounds=rebounds,
            insides=insides,
            clearances=clearances,
            clangers=clangers,
            free_kicks_for=free_kicks_for,
            free_kicks_against=free_kicks_against,
            brownlow_votes=brownlow_votes,
            contested_possessions=contested_possessions,
            uncontested_possessions=uncontested_possessions,
            contested_marks=contested_marks,
            marks_inside=marks_inside,
            one_percenters=one_percenters,
            bounces=bounces,
            goal_assists=goal_assists,
            percentage_played=percentage_played,
            session=session,
            dt=dt,
        )



"""Tests for the decrypt functions."""
import os
import unittest

import requests_mock
import requests_cache
from bs4 import BeautifulSoup
from sportsball.data.oddsportal.decrypt import fetch_data


class TestDecrypt(unittest.TestCase):

    def setUp(self):
        self.session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_decrypt(self):
        referer_url = "https://www.oddsportal.com/aussie-rules/australia/afl-2022/sydney-swans-collingwood-magpies-SnAeelt9/"
        url = "https://www.oddsportal.com/match-event/1-18-SnAeelt9-3-1-yj021.dat?geo=AE&lang=en"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "sydney-swans-collingwood-magpies-SnAeelt9.html"), "rb") as f:
                soup = BeautifulSoup(f.read(), "lxml")
            with open(os.path.join(self.dir, "app_250213122553.js"), "rb") as f:
                m.get("https://www.oddsportal.com/res/public/js/build/app.js?v=250213122553", content=f.read())
            with open(os.path.join(self.dir, "1-18-SnAeelt9-3-1-yj021.dat"), "rb") as f:
                m.get("https://www.oddsportal.com/match-event/1-18-SnAeelt9-3-1-yj021.dat?geo=AE&lang=en", content=f.read())
            data = fetch_data(url, self.session, referer_url, soup)
            self.assertTrue(data)



"""Tests for the afltables game model class."""
import unittest
import os
import datetime

import requests_cache
import requests_mock
from sportsball.data.afl.afltables.afl_afltables_game_model import _create_afl_afltables_game_model
from sportsball.data.league import League


class TestAFLTablesGameModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_player_identifier(self):
        url = "https://afltables.com/afl/stats/games/2025/081820250330.html"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "081820250330.html"), "rb") as f:
                m.get(url, content=f.read())
            with open(os.path.join(self.dir, "perth.html"), "rb") as f:
                m.get("https://afltables.com/afl/venues/perth.html", content=f.read())
            with open(os.path.join(self.dir, "westcoast_idx.html"), "rb") as f:
                m.get("https://afltables.com/afl/teams/westcoast_idx.html", content=f.read())
            with open(os.path.join(self.dir, "fremantle_idx.html"), "rb") as f:
                m.get("https://afltables.com/afl/teams/fremantle_idx.html", content=f.read())
            with open(os.path.join(self.dir, "Oscar_Allen.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/O/Oscar_Allen.html", content=f.read())
            with open(os.path.join(self.dir, "Liam_Baker.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/L/Liam_Baker.html", content=f.read())
            with open(os.path.join(self.dir, "Tyler_Brockman.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/T/Tyler_Brockman.html", content=f.read())
            with open(os.path.join(self.dir, "Tom_Cole.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/T/Tom_Cole.html", content=f.read())
            with open(os.path.join(self.dir, "Jamie_Cripps.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jamie_Cripps.html", content=f.read())
            with open(os.path.join(self.dir, "Tyrell_Dewar.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/T/Tyrell_Dewar.html", content=f.read())
            with open(os.path.join(self.dir, "Liam_Duggan.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/L/Liam_Duggan.html", content=f.read())
            with open(os.path.join(self.dir, "Harry_Edwards.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/H/Harry_Edwards.html", content=f.read())
            with open(os.path.join(self.dir, "Reuben_Ginbey.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/R/Reuben_Ginbey.html", content=f.read())
            with open(os.path.join(self.dir, "Jack_Graham.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jack_Graham.html", content=f.read())
            with open(os.path.join(self.dir, "Elijah_Hewett.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/E/Elijah_Hewett.html", content=f.read())
            with open(os.path.join(self.dir, "Brady_Hough.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/B/Brady_Hough.html", content=f.read())
            with open(os.path.join(self.dir, "Jayden_Hunt.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jayden_Hunt.html", content=f.read())
            with open(os.path.join(self.dir, "Tim_Kelly.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/T/Tim_Kelly.html", content=f.read())
            with open(os.path.join(self.dir, "Noah_Long.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/N/Noah_Long.html", content=f.read())
            with open(os.path.join(self.dir, "Ryan_Maric.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/R/Ryan_Maric.html", content=f.read())
            with open(os.path.join(self.dir, "Jeremy_McGovern.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jeremy_McGovern.html", content=f.read())
            with open(os.path.join(self.dir, "Matthew_Owies.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/M/Matthew_Owies.html", content=f.read())
            with open(os.path.join(self.dir, "Archer_Reid.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/A/Archer_Reid.html", content=f.read())
            with open(os.path.join(self.dir, "Harley_Reid.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/H/Harley_Reid.html", content=f.read())
            with open(os.path.join(self.dir, "Liam_Ryan.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/L/Liam_Ryan.html", content=f.read())
            with open(os.path.join(self.dir, "Bailey_Williams1.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/B/Bailey_Williams1.html", content=f.read())
            with open(os.path.join(self.dir, "Jack_Williams.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jack_Williams.html", content=f.read())
            with open(os.path.join(self.dir, "James_Aish.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/James_Aish.html", content=f.read())
            with open(os.path.join(self.dir, "Jye_Amiss.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jye_Amiss.html", content=f.read())
            with open(os.path.join(self.dir, "Bailey_Banfield.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/B/Bailey_Banfield.html", content=f.read())
            with open(os.path.join(self.dir, "Shai_Bolton.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/S/Shai_Bolton.html", content=f.read())
            with open(os.path.join(self.dir, "Andrew_Brayshaw.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/A/Andrew_Brayshaw.html", content=f.read())
            with open(os.path.join(self.dir, "Heath_Chapman.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/H/Heath_Chapman.html", content=f.read())
            with open(os.path.join(self.dir, "Jordan_Clark.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jordan_Clark.html", content=f.read())
            with open(os.path.join(self.dir, "Brennan_Cox.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/B/Brennan_Cox.html", content=f.read())
            with open(os.path.join(self.dir, "Josh_Draper.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Josh_Draper.html", content=f.read())
            with open(os.path.join(self.dir, "Isaiah_Dudley.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/I/Isaiah_Dudley.html", content=f.read())
            with open(os.path.join(self.dir, "Neil_Erasmus.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/N/Neil_Erasmus.html", content=f.read())
            with open(os.path.join(self.dir, "Michael_Frederick.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/M/Michael_Frederick.html", content=f.read())
            with open(os.path.join(self.dir, "Luke_Jackson.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/L/Luke_Jackson.html", content=f.read())
            with open(os.path.join(self.dir, "Matthew_Johnson.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/M/Matthew_Johnson.html", content=f.read())
            with open(os.path.join(self.dir, "Nathan_ODriscoll.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/N/Nathan_ODriscoll.html", content=f.read())
            with open(os.path.join(self.dir, "Alex_Pearce.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/A/Alex_Pearce.html", content=f.read())
            with open(os.path.join(self.dir, "Murphy_Reid.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/M/Murphy_Reid.html", content=f.read())
            with open(os.path.join(self.dir, "Luke_Ryan.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/L/Luke_Ryan.html", content=f.read())
            with open(os.path.join(self.dir, "Caleb_Serong.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/C/Caleb_Serong.html", content=f.read())
            with open(os.path.join(self.dir, "Jeremy_Sharp.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Jeremy_Sharp.html", content=f.read())
            with open(os.path.join(self.dir, "Josh_Treacy.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/J/Josh_Treacy.html", content=f.read())
            with open(os.path.join(self.dir, "Patrick_Voss.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/P/Patrick_Voss.html", content=f.read())
            with open(os.path.join(self.dir, "Karl_Worner.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/players/K/Karl_Worner.html", content=f.read())
            with open(os.path.join(self.dir, "coaches_westcoast.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/westcoast.html", content=f.read())
            with open(os.path.join(self.dir, "coaches_fremantle.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/fremantle.html", content=f.read())
            with open(os.path.join(self.dir, "John_Worsfold.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/John_Worsfold.html", content=f.read())
            with open(os.path.join(self.dir, "Ross_Lyon.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/Ross_Lyon.html", content=f.read())
            with open(os.path.join(self.dir, "Andrew_McQualter.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/Andrew_McQualter.html", content=f.read())
            with open(os.path.join(self.dir, "Justin_Longmuir.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/Justin_Longmuir.html", content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=-31.9511597&longitude=115.884175&start_date=2025-03-29&end_date=2025-03-30&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=Australia%2FPerth&format=flatbuffers")

            game_model = _create_afl_afltables_game_model(
                0,
                self._session,
                url,
                2,
                {},
                League.AFL,
                2025,
                season_type=None,
            )
            self.assertIsNotNone(game_model.teams[0].players[0].identifier)
            self.assertEqual(game_model.dt, datetime.datetime(2025, 3, 30, 15, 10))



"""HKJC HKJC dividend model."""

from ...bet import bet_from_str
from ...dividend_model import DividendModel
from ...team_model import TeamModel


def create_hkjc_hkjc_dividend_model(
    pool: str, combination: list[str], team_models: list[TeamModel], dividend: float
) -> DividendModel:
    """Create a dividend model from HKJC."""
    jerseys_teams = {
        x.players[0].jersey: x.identifier for x in team_models if x.players
    }
    return DividendModel(
        pool=bet_from_str(pool),
        combination=[jerseys_teams[x] for x in combination if x in jerseys_teams],
        dividend=dividend,
    )



"""Tests for the combined team model class."""
import datetime
import os
import unittest

import requests_mock
import requests_cache
from sportsball.data.combined.combined_team_model import create_combined_team_model
from sportsball.data.team_model import TeamModel
from sportsball.data.player_model import PlayerModel
from sportsball.data.species import Species


class TestCombinedTeamModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_names_resolve(self):
        with requests_mock.Mocker() as m:
            names = {}
            coach_names = {}
            player_models = [PlayerModel(
                identifier="1",
                jersey=None,
                kicks=None,
                fumbles=None,
                fumbles_lost=None,
                field_goals=None,
                field_goals_attempted=None,
                offensive_rebounds=None,
                assists=None,
                turnovers=None,
                name="Johnny Johnson",
                marks=None,
                handballs=None,
                disposals=None,
                goals=None,
                behinds=None,
                hit_outs=None,
                tackles=None,
                rebounds=None,
                insides=None,
                clearances=None,
                clangers=None,
                free_kicks_for=None,
                free_kicks_against=None,
                brownlow_votes=None,
                contested_possessions=None,
                uncontested_possessions=None,
                contested_marks=None,
                marks_inside=None,
                one_percenters=None,
                bounces=None,
                goal_assists=None,
                percentage_played=None,
                birth_date=None,
                age=None,
                species=str(Species.HUMAN),
                handicap_weight=None,
                father=None,
                starting_position=None,
                weight=None,
                birth_address=None,
                owner=None,
            )]
            team_models = [TeamModel(
                identifier="a",
                name="Team A",
                location=None,
                players=player_models,
                odds=[],
                points=None,
                ladder_rank=None,
                news=[],
                social=[],
                coaches=[],
                lbw=None,
                end_dt=None,
            )]
            team_model = create_combined_team_model(
                team_models=team_models,
                identifier="a",
                player_identity_map={},
                names=names,
                coach_names=coach_names,
            )
            player_models_2 = [PlayerModel(
                identifier="1a",
                jersey=None,
                kicks=None,
                fumbles=None,
                fumbles_lost=None,
                field_goals=None,
                field_goals_attempted=None,
                offensive_rebounds=None,
                assists=None,
                turnovers=None,
                name="Johnny Johnson",
                marks=None,
                handballs=None,
                disposals=None,
                goals=None,
                behinds=None,
                hit_outs=None,
                tackles=None,
                rebounds=None,
                insides=None,
                clearances=None,
                clangers=None,
                free_kicks_for=None,
                free_kicks_against=None,
                brownlow_votes=None,
                contested_possessions=None,
                uncontested_possessions=None,
                contested_marks=None,
                marks_inside=None,
                one_percenters=None,
                bounces=None,
                goal_assists=None,
                percentage_played=None,
                birth_date=None,
                age=None,
                species=str(Species.HUMAN),
                handicap_weight=None,
                father=None,
                starting_position=None,
                weight=None,
                birth_address=None,
                owner=None,
            )]
            team_models_2 = [TeamModel(
                identifier="a",
                name="Team A",
                location=None,
                players=player_models_2,
                odds=[],
                points=None,
                ladder_rank=None,
                news=[],
                social=[],
                coaches=[],
                lbw=None,
                end_dt=None,
            )]
            team_model_2 = create_combined_team_model(
                team_models=team_models_2,
                identifier="a",
                player_identity_map={},
                names=names,
                coach_names=coach_names,
            )
            self.assertEqual(team_model.players[0].identifier, team_model_2.players[0].identifier)

    def test_names_resolve_with_surname_firstname(self):
        with requests_mock.Mocker() as m:
            names = {}
            coach_names = {}
            player_models = [PlayerModel(
                identifier="1",
                jersey=None,
                kicks=None,
                fumbles=None,
                fumbles_lost=None,
                field_goals=None,
                field_goals_attempted=None,
                offensive_rebounds=None,
                assists=None,
                turnovers=None,
                name="Johnny Johnson",
                marks=None,
                handballs=None,
                disposals=None,
                goals=None,
                behinds=None,
                hit_outs=None,
                tackles=None,
                rebounds=None,
                insides=None,
                clearances=None,
                clangers=None,
                free_kicks_for=None,
                free_kicks_against=None,
                brownlow_votes=None,
                contested_possessions=None,
                uncontested_possessions=None,
                contested_marks=None,
                marks_inside=None,
                one_percenters=None,
                bounces=None,
                goal_assists=None,
                percentage_played=None,
                birth_date=None,
                age=None,
                species=str(Species.HUMAN),
                handicap_weight=None,
                father=None,
                starting_position=None,
                weight=None,
                birth_address=None,
                owner=None,
            )]
            team_models = [TeamModel(
                identifier="a",
                name="Team A",
                location=None,
                players=player_models,
                odds=[],
                points=None,
                ladder_rank=None,
                news=[],
                social=[],
                coaches=[],
                lbw=None,
                end_dt=None,
            )]
            team_model = create_combined_team_model(
                team_models=team_models,
                identifier="a",
                player_identity_map={},
                names=names,
                coach_names=coach_names,
            )
            player_models_2 = [PlayerModel(
                identifier="1a",
                jersey=None,
                kicks=None,
                fumbles=None,
                fumbles_lost=None,
                field_goals=None,
                field_goals_attempted=None,
                offensive_rebounds=None,
                assists=None,
                turnovers=None,
                name="Johnson, Johnny",
                marks=None,
                handballs=None,
                disposals=None,
                goals=None,
                behinds=None,
                hit_outs=None,
                tackles=None,
                rebounds=None,
                insides=None,
                clearances=None,
                clangers=None,
                free_kicks_for=None,
                free_kicks_against=None,
                brownlow_votes=None,
                contested_possessions=None,
                uncontested_possessions=None,
                contested_marks=None,
                marks_inside=None,
                one_percenters=None,
                bounces=None,
                goal_assists=None,
                percentage_played=None,
                birth_date=None,
                age=None,
                species=str(Species.HUMAN),
                handicap_weight=None,
                father=None,
                starting_position=None,
                weight=None,
                birth_address=None,
                owner=None,
            )]
            team_models_2 = [TeamModel(
                identifier="a",
                name="Team A",
                location=None,
                players=player_models_2,
                odds=[],
                points=None,
                ladder_rank=None,
                news=[],
                social=[],
                coaches=[],
                lbw=None,
                end_dt=None,
            )]
            team_model_2 = create_combined_team_model(
                team_models=team_models_2,
                identifier="a",
                player_identity_map={},
                names=names,
                coach_names=coach_names,
            )
            self.assertEqual(team_model.players[0].identifier, team_model_2.players[0].identifier)



"""The prototype class for a player."""

from __future__ import annotations

import datetime
from typing import Any, Literal

import gender_guesser.detector as gender  # type: ignore
from pydantic import BaseModel, Field

from .address_model import AddressModel
from .field_type import TYPE_KEY, FieldType
from .owner_model import OwnerModel
from .sex import Sex

PLAYER_KICKS_COLUMN: Literal["kicks"] = "kicks"
PLAYER_IDENTIFIER_COLUMN: Literal["identifier"] = "identifier"
PLAYER_FUMBLES_COLUMN: Literal["fumbles"] = "fumbles"
PLAYER_FUMBLES_LOST_COLUMN: Literal["fumbles_lost"] = "fumbles_lost"
FIELD_GOALS_COLUMN: Literal["field_goals"] = "field_goals"
FIELD_GOALS_ATTEMPTED_COLUMN: Literal["field_goals_attempted"] = "field_goals_attempted"
OFFENSIVE_REBOUNDS_COLUMN: Literal["offensive_rebounds"] = "offensive_rebounds"
ASSISTS_COLUMN: Literal["assists"] = "assists"
TURNOVERS_COLUMN: Literal["turnovers"] = "turnovers"
PLAYER_MARKS_COLUMN: Literal["marks"] = "marks"
PLAYER_HANDBALLS_COLUMN: Literal["handballs"] = "handballs"
PLAYER_DISPOSALS_COLUMN: Literal["disposals"] = "disposals"
PLAYER_GOALS_COLUMN: Literal["goals"] = "goals"
PLAYER_BEHINDS_COLUMN: Literal["behinds"] = "behinds"
PLAYER_HIT_OUTS_COLUMN: Literal["hit_outs"] = "hit_outs"
PLAYER_TACKLES_COLUMN: Literal["tackles"] = "tackles"
PLAYER_REBOUNDS_COLUMN: Literal["rebounds"] = "rebounds"
PLAYER_INSIDES_COLUMN: Literal["insides"] = "insides"
PLAYER_CLEARANCES_COLUMN: Literal["clearances"] = "clearances"
PLAYER_CLANGERS_COLUMN: Literal["clangers"] = "clangers"
PLAYER_FREE_KICKS_FOR_COLUMN: Literal["free_kicks_for"] = "free_kicks_for"
PLAYER_FREE_KICKS_AGAINST_COLUMN: Literal["free_kicks_against"] = "free_kicks_against"
PLAYER_BROWNLOW_VOTES_COLUMN: Literal["brownlow_votes"] = "brownlow_votes"
PLAYER_CONTESTED_POSSESSIONS_COLUMN: Literal["contested_possessions"] = (
    "contested_possessions"
)
PLAYER_UNCONTESTED_POSSESSIONS_COLUMN: Literal["uncontested_possessions"] = (
    "uncontested_possessions"
)
PLAYER_CONTESTED_MARKS_COLUMN: Literal["contested_marks"] = "contested_marks"
PLAYER_MARKS_INSIDE_COLUMN: Literal["marks_inside"] = "marks_inside"
PLAYER_ONE_PERCENTERS_COLUMN: Literal["one_percenters"] = "one_percenters"
PLAYER_BOUNCES_COLUMN: Literal["bounces"] = "bounces"
PLAYER_GOAL_ASSISTS_COLUMN: Literal["goal_assists"] = "goal_assists"
PLAYER_PERCENTAGE_PLAYED_COLUMN: Literal["percentage_played"] = "percentage_played"
PLAYER_NAME_COLUMN: Literal["name"] = "name"
PLAYER_BIRTH_DATE_COLUMN: Literal["birth_date"] = "birth_date"
PLAYER_SPECIES_COLUMN: Literal["species"] = "species"
PLAYER_HANDICAP_WEIGHT_COLUMN: Literal["handicap_weight"] = "handicap_weight"
PLAYER_FATHER_COLUMN: Literal["father"] = "father"
PLAYER_SEX_COLUMN: Literal["sex"] = "sex"
PLAYER_AGE_COLUMN: Literal["age"] = "age"
PLAYER_STARTING_POSITION_COLUMN: Literal["starting_position"] = "starting_position"
PLAYER_WEIGHT_COLUMN: Literal["weight"] = "weight"
PLAYER_BIRTH_ADDRESS_COLUMN: Literal["birth_address"] = "birth_address"
PLAYER_OWNER_COLUMN: Literal["owner"] = "owner"

_GENDER_DETECTOR = gender.Detector()
_MALE_GENDERS = {"male", "mostly_male"}
_FEMALE_GENDERS = {"female", "mostly_female"}
_UNCERTAIN_GENDERS = {"andy", "unknown"}


def _guess_sex(data: dict[str, Any]) -> str | None:
    name = data[PLAYER_NAME_COLUMN]
    gender_tag = _GENDER_DETECTOR.get_gender(name)
    if gender_tag in _MALE_GENDERS:
        return str(Sex.MALE)
    if gender_tag in _FEMALE_GENDERS:
        return str(Sex.FEMALE)
    if gender_tag in _UNCERTAIN_GENDERS:
        return None
    return None


class PlayerModel(BaseModel):
    """The serialisable player class."""

    identifier: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=PLAYER_IDENTIFIER_COLUMN,
    )
    jersey: str | None = Field(..., json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL})
    kicks: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_KICKS_COLUMN,
    )
    fumbles: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_FUMBLES_COLUMN,
    )
    fumbles_lost: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_FUMBLES_LOST_COLUMN,
    )
    field_goals: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=FIELD_GOALS_COLUMN,
    )
    field_goals_attempted: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=FIELD_GOALS_ATTEMPTED_COLUMN,
    )
    offensive_rebounds: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=OFFENSIVE_REBOUNDS_COLUMN,
    )
    assists: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=ASSISTS_COLUMN,
    )
    turnovers: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=TURNOVERS_COLUMN,
    )
    name: str = Field(..., alias=PLAYER_NAME_COLUMN)
    marks: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_MARKS_COLUMN,
    )
    handballs: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_HANDBALLS_COLUMN,
    )
    disposals: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_DISPOSALS_COLUMN,
    )
    goals: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_GOALS_COLUMN,
    )
    behinds: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_BEHINDS_COLUMN,
    )
    hit_outs: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_HIT_OUTS_COLUMN,
    )
    tackles: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_TACKLES_COLUMN,
    )
    rebounds: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_REBOUNDS_COLUMN,
    )
    insides: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_INSIDES_COLUMN,
    )
    clearances: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_CLEARANCES_COLUMN,
    )
    clangers: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_CLANGERS_COLUMN,
    )
    free_kicks_for: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_FREE_KICKS_FOR_COLUMN,
    )
    free_kicks_against: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_FREE_KICKS_AGAINST_COLUMN,
    )
    brownlow_votes: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_BROWNLOW_VOTES_COLUMN,
    )
    contested_possessions: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_CONTESTED_POSSESSIONS_COLUMN,
    )
    uncontested_possessions: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_UNCONTESTED_POSSESSIONS_COLUMN,
    )
    contested_marks: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_CONTESTED_MARKS_COLUMN,
    )
    marks_inside: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_MARKS_INSIDE_COLUMN,
    )
    one_percenters: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_ONE_PERCENTERS_COLUMN,
    )
    bounces: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_BOUNCES_COLUMN,
    )
    goal_assists: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_GOAL_ASSISTS_COLUMN,
    )
    percentage_played: float | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=PLAYER_PERCENTAGE_PLAYED_COLUMN,
    )
    birth_date: datetime.date | None = Field(..., alias=PLAYER_BIRTH_DATE_COLUMN)
    species: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=PLAYER_SPECIES_COLUMN,
    )
    handicap_weight: float | None = Field(..., alias=PLAYER_HANDICAP_WEIGHT_COLUMN)
    father: PlayerModel | None = Field(..., alias=PLAYER_FATHER_COLUMN)
    sex: str | None = Field(
        default_factory=_guess_sex,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=PLAYER_SEX_COLUMN,
    )
    age: int | None = Field(..., alias=PLAYER_AGE_COLUMN)
    starting_position: str | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=PLAYER_STARTING_POSITION_COLUMN,
    )
    weight: float | None = Field(..., alias=PLAYER_WEIGHT_COLUMN)
    birth_address: AddressModel | None = Field(..., alias=PLAYER_BIRTH_ADDRESS_COLUMN)
    owner: OwnerModel | None = Field(..., alias=PLAYER_OWNER_COLUMN)



"""Odds Portal venue model."""

import datetime

import pytest_is_running
import requests_cache

from ...cache import MEMORY
from ..google.google_address_model import create_google_address_model
from ..venue_model import VenueModel


def _create_oddsportal_venue_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
) -> VenueModel | None:
    return VenueModel(
        identifier=venue,
        name=venue,
        address=create_google_address_model(
            ", ".join([venue, venue_town, venue_country]), session, dt
        ),
        is_grass=None,
        is_indoor=None,
        is_turf=None,
        is_dirt=None,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_oddsportal_venue_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
) -> VenueModel | None:
    return _create_oddsportal_venue_model(session, dt, venue, venue_town, venue_country)


def create_oddsportal_venue_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
) -> VenueModel | None:
    """Create odds portal venue model."""
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=7):
        return _cached_create_oddsportal_venue_model(
            session, dt, venue, venue_town, venue_country
        )
    with session.cache_disabled():
        return _create_oddsportal_venue_model(
            session, dt, venue, venue_town, venue_country
        )



"""The enumeration of the different supported leagues."""

from enum import StrEnum


class League(StrEnum):
    """An enumeration over the different leagues."""

    AFL = "afl"
    HKJC = "hkjc"
    NBA = "nba"
    NCAAB = "ncaab"
    NCAAF = "ncaaf"
    NFL = "nfl"


def long_name(league: League) -> str:
    """Find the leagues long name."""
    match league:
        case League.AFL:
            return "Australia Football League"
        case League.HKJC:
            return "Hong Kong Jockey Club"
        case League.NBA:
            return "National Basketball League"
        case League.NCAAB:
            return "NCAA Division I Basketball"
        case League.NCAAF:
            return "NCAA Division I Football"
        case League.NFL:
            return "National Football League"


def league_from_str(league_str: str) -> League:
    """Find the league matching the string."""
    league_str = league_str.lower()
    match league_str:
        case str(League.AFL):
            return League.AFL
        case str(League.HKJC):
            return League.HKJC
        case str(League.NBA):
            return League.NBA
        case str(League.NCAAB):
            return League.NCAAB
        case str(League.NCAAF):
            return League.NCAAF
        case str(League.NFL):
            return League.NFL
        case _:
            raise ValueError(f"Unrecognised League: {league_str}")



"""NBA combined league model."""

# pylint: disable=line-too-long

import requests_cache

from ...combined.combined_league_model import CombinedLeagueModel
from ...league import League
from ..espn.nba_espn_league_model import NBAESPNLeagueModel
from ..nba.nba_nba_league_model import NBANBALeagueModel
from ..nbacom.nba_nbacom_league_model import NBANBAComLeagueModel
from ..oddsportal.nba_oddsportal_league_model import NBAOddsPortalLeagueModel
from ..sportsdb.nba_sportsdb_league_model import NBASportsDBLeagueModel
from ..sportsreference.nba_sportsreference_league_model import \
    NBASportsReferenceLeagueModel

ORLANDO_MAGIC = "1610612753"
ATLANTA_HAWKS = "1610612737"
PORTLAND_TRAIL_BLAZERS = "1610612757"
DENVER_NUGGETS = "1610612743"
MIAMI_HEAT = "1610612748"
LOS_ANGELES_LAKERS = "1610612747"
BOSTON_CELTICS = "1610612738"
OKLAHOMA_CITY_THUNDER = "1610612760"
SACRAMENTO_KINGS = "1610612758"
INDIANA_PACERS = "1610612754"
SAN_ANTONIO_SPURS = "1610612759"
PHOENIX_SUNS = "1610612756"
WASHINGTON_WIZARDS = "1610612764"
PHILADELPHIA_76ERS = "1610612755"
TORONTO_RAPTORS = "1610612761"
GOLDEN_STATE_WARRIORS = "1610612744"
MINNESOTA_TIMBERWOLVES = "1610612750"
HOUSTON_ROCKETS = "1610612745"
MEMPHIS_GRIZZLIES = "1610612763"
NEW_YORK_KNICKS = "1610612752"
CHARLOTTE_HORNETS = "1610612766"
BROOKLYN_NETS = "1610612751"
DALLAS_MAVERICKS = "1610612742"
CLEVELAND_CAVALIERS = "1610612739"
MILWAUKEE_BUCKS = "1610612749"
DETROIT_PISTONS = "1610612765"
NEW_ORLEANS_PELICANS = "1610612740"
CHICAGO_BULLS = "1610612741"
LA_CLIPPERS = "1610612746"
UTAH_JAZZ = "1610612762"
UNKNOWN = "1610616842"
TEAM_JALEN = "1610616849"
TEAM_TAMIKA = "1610616850"
TEAM_PAU = "1610616847"
TEAM_DETLEF = "1610616848"
TEAM_STEPHEN_A = "1610616840"
EAST_NBA_ALL_STARS = "1610616833"
WEST_NBA_ALL_STARS = "1610616834"
TEAM_SHANNON = "1610616839"
NEW_ZEALAND_BREAKERS = "15020"
RATIOPHARM_ULM = "15025"
CAIRNS_TAIPANS = "15022"
RAANANA_MACCABI = "50009"
FLAMENGO = "12325"
MADRID_BALONCESTO = "12315"
ADELAIDE_36ERS = "15019"
TEAM_USA = "1610616843"
SHANGHAI_SHARKS = "12329"
SESI_FRANCA_BASKETBALL_CLUB_OF_BRAZIL = "12332"
HAIFA_MACABI = "93"
BUENOS_AIRES_SAN_LORENZO = "12330"
GUANGZHOU_LONG_LIONS = "15018"
MELBOURNE_UNITED = "15016"
WORLD = "1610616844"
BEIJING_DUCKS = "15021"
PERTH_WILDCATS = "104"
SYDNEY_KINGS = "15015"
BRISBANE_BULLETS = "15017"
BARCELONA_LASSA = "12304"
ISTANBUL_FENERBAHCE_SPORTS_CLUB = "12321"
MILANO_OLIMPIA = "94"
BAURU_PASCHOALOTTO = "12328"
TEL_AVIV_MACCABI_ELECTRA = "41"
BERLIN_ALBA = "12323"
BILBAO_BASKET = "12324"
MOSCOW_CSKA = "12308"
SIENA_MONTEPASCI = "12322"
SOPHOMORES = "1610616838"
ROOKIES = "1610616837"
VITORIA_CAJAL_LABORAL = "95"
BELGRADE_PARTIZAN = "12319"
ATHENS_OLYMPIACOS = "12320"
HORNETS = "1610612778"
VILNIUS_LIETUVOS_RYTAS = "12314"
ISTANBUL_EFES_PILSEN = "12309"
LOTTOMATICA_ROMA = "12317"
UNICAJA_MALAGA = "12311"
MMT_ESTUDIANTES = "12318"
PANATHINAIKOS = "12313"
ZALGIRIS_KAUNAS = "12316"
TEAM_CHINA = "45"
CLUTCH_CITY = "1610612767"
H_TOWN = "1610612778"
LYON_VILLEURBANNE_ADECCO_ASVEL = "12305"
KHIMKI_BC = "12306"
TEL_AVIV_MACCABI_ELITE = "12303"
SEATTLE_SUPERSONICS = "137362"
BUFFALO_BRAVES = "139606"
ST_LOUIS_BOMBERS = "St. Louis Bombers"
PITTSBURGH_IRONMEN = "Pittsburgh Ironmen"
PROVIDENCE_STEAM_ROLLERS = "Providence Steam Rollers"
BALTIMORE_BULLETS = "Baltimore Bullets"
FORT_WAYNE_PISTONS = "Fort Wayne Pistons"
ROCHESTER_ROYALS = "Rochester Royals"
TRICITIES_BLACKHAWKS = "Tri-Cities Blackhawks"
SHEBOYGAN_REDSKINS = "Sheboygan Red Skins"
WATERLOO_HAWKS = "Waterloo Hawks"
SYRACUSE_NATIONALS = "Syracuse Nationals"
ANDERSON_PACKERS = "Anderson Packers"
CINCINATTI_ROYALS = "Cincinnati Royals"
SAN_FRANCISCO_WARRIORS = "San Francisco Warriors"
OAKLAND_OAKS = "Oakland Oaks"
ANAHEIM_AMIGOS = "Anaheim Amigos"
KENTUCKY_COLONELS = "Kentucky Colonels"
SAN_DIEGO_ROCKETS = "San Diego Rockets"
NEW_JERSEY_AMERICANS = "New Jersey Americans"
CAROLINA_COUGARS = "Carolina Cougars"
VIRGINIA_SQUIRES = "Virginia Squires"
KANSAS_CITY_KINGS = "Kansas City-Omaha Kings"
CAPITAL_BULLETS = "Capital Bullets"
VANCOUVER_GRIZZLIES = "Vancouver Grizzlies"
OGS = "1610616852"
GLOBAL_STARS = "1610616851"
RISING_STARS = "1610616854"
YOUNG_STARS = "1610616853"
TEAM_G_LEAGUE = "1610616858"
TEAM_C = "1610616855"
TEAM_M = "1610616857"
TEAM_T = "1610616856"
NBA_TEAM_IDENTITY_MAP = {
    # NBA API
    "1610612753": ORLANDO_MAGIC,
    "1610612737": ATLANTA_HAWKS,
    "1610612757": PORTLAND_TRAIL_BLAZERS,
    "1610612743": DENVER_NUGGETS,
    "1610612748": MIAMI_HEAT,
    "1610612747": LOS_ANGELES_LAKERS,
    "1610612738": BOSTON_CELTICS,
    "1610612760": OKLAHOMA_CITY_THUNDER,
    "1610612758": SACRAMENTO_KINGS,
    "1610612754": INDIANA_PACERS,
    "1610612759": SAN_ANTONIO_SPURS,
    "1610612756": PHOENIX_SUNS,
    "1610612764": WASHINGTON_WIZARDS,
    "1610612755": PHILADELPHIA_76ERS,
    "1610612761": TORONTO_RAPTORS,
    "1610612744": GOLDEN_STATE_WARRIORS,
    "1610612750": MINNESOTA_TIMBERWOLVES,
    "1610612745": HOUSTON_ROCKETS,
    "1610612763": MEMPHIS_GRIZZLIES,
    "1610612752": NEW_YORK_KNICKS,
    "1610612766": CHARLOTTE_HORNETS,
    "1610612751": BROOKLYN_NETS,
    "1610612742": DALLAS_MAVERICKS,
    "1610612739": CLEVELAND_CAVALIERS,
    "1610612749": MILWAUKEE_BUCKS,
    "1610612765": DETROIT_PISTONS,
    "1610612740": NEW_ORLEANS_PELICANS,
    "1610612741": CHICAGO_BULLS,
    "1610612746": LA_CLIPPERS,
    "1610612762": UTAH_JAZZ,
    "1610616842": UNKNOWN,
    "1610616841": UNKNOWN,
    "1610616849": TEAM_JALEN,
    "1610616850": TEAM_TAMIKA,
    "1610616847": TEAM_PAU,
    "1610616848": TEAM_DETLEF,
    "1610616840": TEAM_STEPHEN_A,
    "1610616833": EAST_NBA_ALL_STARS,
    "1610616834": WEST_NBA_ALL_STARS,
    "1610616839": TEAM_SHANNON,
    "1610616852": OGS,
    "1610616851": GLOBAL_STARS,
    "1610616854": RISING_STARS,
    "15020": NEW_ZEALAND_BREAKERS,
    "15025": RATIOPHARM_ULM,
    "15022": CAIRNS_TAIPANS,
    "50009": RAANANA_MACCABI,
    "12325": FLAMENGO,
    "12315": MADRID_BALONCESTO,
    "15019": ADELAIDE_36ERS,
    "1610616843": TEAM_USA,
    "12329": SHANGHAI_SHARKS,
    "12332": SESI_FRANCA_BASKETBALL_CLUB_OF_BRAZIL,
    "93": HAIFA_MACABI,
    "12330": BUENOS_AIRES_SAN_LORENZO,
    "15018": GUANGZHOU_LONG_LIONS,
    "15016": MELBOURNE_UNITED,
    "1610616844": WORLD,
    "15021": BEIJING_DUCKS,
    "104": PERTH_WILDCATS,
    "15015": SYDNEY_KINGS,
    "15017": BRISBANE_BULLETS,
    "12304": BARCELONA_LASSA,
    "12321": ISTANBUL_FENERBAHCE_SPORTS_CLUB,
    "94": MILANO_OLIMPIA,
    "12328": BAURU_PASCHOALOTTO,
    "41": TEL_AVIV_MACCABI_ELECTRA,
    "12323": BERLIN_ALBA,
    "12324": BILBAO_BASKET,
    "12308": MOSCOW_CSKA,
    "12322": SIENA_MONTEPASCI,
    "1610616838": SOPHOMORES,
    "1610616837": ROOKIES,
    "95": VITORIA_CAJAL_LABORAL,
    "12319": BELGRADE_PARTIZAN,
    "12320": ATHENS_OLYMPIACOS,
    "1610612778": HORNETS,
    "12314": VILNIUS_LIETUVOS_RYTAS,
    "12309": ISTANBUL_EFES_PILSEN,
    "12317": LOTTOMATICA_ROMA,
    "12311": UNICAJA_MALAGA,
    "12318": MMT_ESTUDIANTES,
    "12313": PANATHINAIKOS,
    "12316": ZALGIRIS_KAUNAS,
    "45": TEAM_CHINA,
    "1610612767": CLUTCH_CITY,
    "12305": LYON_VILLEURBANNE_ADECCO_ASVEL,
    "12306": KHIMKI_BC,
    "12303": TEL_AVIV_MACCABI_ELITE,
    "12307": LOTTOMATICA_ROMA,
    "1610616853": YOUNG_STARS,
    "1610616858": TEAM_G_LEAGUE,
    "1610616855": TEAM_C,
    "1610616857": TEAM_M,
    "1610616856": TEAM_T,
    # SportsDB
    "134867": LOS_ANGELES_LAKERS,
    "134872": DETROIT_PISTONS,
    "134865": GOLDEN_STATE_WARRIORS,
    "134863": PHILADELPHIA_76ERS,
    "134860": BOSTON_CELTICS,
    "134880": ATLANTA_HAWKS,
    "134869": SACRAMENTO_KINGS,
    "134862": NEW_YORK_KNICKS,
    "134884": WASHINGTON_WIZARDS,
    "134870": CHICAGO_BULLS,
    "137362": SEATTLE_SUPERSONICS,
    "134876": HOUSTON_ROCKETS,
    "134874": MILWAUKEE_BUCKS,
    "134868": PHOENIX_SUNS,
    "134871": CLEVELAND_CAVALIERS,
    "139606": BUFFALO_BRAVES,
    "134888": PORTLAND_TRAIL_BLAZERS,
    "134889": UTAH_JAZZ,
    "134885": DENVER_NUGGETS,
    "134879": SAN_ANTONIO_SPURS,
    "134873": INDIANA_PACERS,
    "134861": BROOKLYN_NETS,
    "134866": LA_CLIPPERS,
    "134875": DALLAS_MAVERICKS,
    "134881": CHARLOTTE_HORNETS,
    "134882": MIAMI_HEAT,
    "134886": MINNESOTA_TIMBERWOLVES,
    "134883": ORLANDO_MAGIC,
    "134887": OKLAHOMA_CITY_THUNDER,
    "134877": MEMPHIS_GRIZZLIES,
    "134864": TORONTO_RAPTORS,
    "134878": NEW_ORLEANS_PELICANS,
    "135398": NEW_ZEALAND_BREAKERS,
    # SportsReference
    "Los Angeles Lakers": LOS_ANGELES_LAKERS,
    "Milwaukee Bucks": MILWAUKEE_BUCKS,
    "Philadelphia 76ers": PHILADELPHIA_76ERS,
    "Houston Rockets": HOUSTON_ROCKETS,
    "Los Angeles Clippers": LA_CLIPPERS,
    "Dallas Mavericks": DALLAS_MAVERICKS,
    "Indiana Pacers": INDIANA_PACERS,
    "Washington Wizards": WASHINGTON_WIZARDS,
    "Boston Celtics": BOSTON_CELTICS,
    "Toronto Raptors": TORONTO_RAPTORS,
    "Charlotte Hornets": CHARLOTTE_HORNETS,
    "Orlando Magic": ORLANDO_MAGIC,
    "Miami Heat": MIAMI_HEAT,
    "Detroit Pistons": DETROIT_PISTONS,
    "Cleveland Cavaliers": CLEVELAND_CAVALIERS,
    "Brooklyn Nets": BROOKLYN_NETS,
    "Sacramento Kings": SACRAMENTO_KINGS,
    "Oklahoma City Thunder": OKLAHOMA_CITY_THUNDER,
    "Utah Jazz": UTAH_JAZZ,
    "New Orleans Pelicans": NEW_ORLEANS_PELICANS,
    "Minnesota Timberwolves": MINNESOTA_TIMBERWOLVES,
    "Portland Trail Blazers": PORTLAND_TRAIL_BLAZERS,
    "Chicago Bulls": CHICAGO_BULLS,
    "San Antonio Spurs": SAN_ANTONIO_SPURS,
    "Denver Nuggets": DENVER_NUGGETS,
    "New York Knicks": NEW_YORK_KNICKS,
    "Phoenix Suns": PHOENIX_SUNS,
    "Memphis Grizzlies": MEMPHIS_GRIZZLIES,
    "Golden State Warriors": GOLDEN_STATE_WARRIORS,
    "Atlanta Hawks": ATLANTA_HAWKS,
    "Toronto Huskies": TORONTO_RAPTORS,
    "St. Louis Bombers": ST_LOUIS_BOMBERS,
    "Pittsburgh Ironmen": PITTSBURGH_IRONMEN,
    "Providence Steam Rollers": PROVIDENCE_STEAM_ROLLERS,
    "Detroit Falcons": DETROIT_PISTONS,
    "Washington Capitols": WASHINGTON_WIZARDS,
    "Chicago Stags": CHICAGO_BULLS,
    "Cleveland Rebels": CLEVELAND_CAVALIERS,
    "Philadelphia Warriors": PHILADELPHIA_76ERS,
    "Baltimore Bullets": BALTIMORE_BULLETS,
    "Indianapolis Jets": INDIANA_PACERS,
    "Fort Wayne Pistons": FORT_WAYNE_PISTONS,
    "Minneapolis Lakers": MINNESOTA_TIMBERWOLVES,
    "Rochester Royals": ROCHESTER_ROYALS,
    "Tri-Cities Blackhawks": TRICITIES_BLACKHAWKS,
    "Sheboygan Red Skins": SHEBOYGAN_REDSKINS,
    "Indianapolis Olympians": INDIANA_PACERS,
    "Waterloo Hawks": WATERLOO_HAWKS,
    "Syracuse Nationals": SYRACUSE_NATIONALS,
    "Anderson Packers": ANDERSON_PACKERS,
    "Milwaukee Hawks": MILWAUKEE_BUCKS,
    "St. Louis Hawks": ST_LOUIS_BOMBERS,
    "Cincinnati Royals": CINCINATTI_ROYALS,
    "Chicago Packers": CHICAGO_BULLS,
    "Chicago Zephyrs": CHICAGO_BULLS,
    "San Francisco Warriors": SAN_FRANCISCO_WARRIORS,
    "Oakland Oaks": OAKLAND_OAKS,
    "Anaheim Amigos": ANAHEIM_AMIGOS,
    "Seattle SuperSonics": SEATTLE_SUPERSONICS,
    "Kentucky Colonels": KENTUCKY_COLONELS,
    "San Diego Rockets": SAN_DIEGO_ROCKETS,
    "Denver Rockets": DENVER_NUGGETS,
    "Dallas Chaparrals": DALLAS_MAVERICKS,
    "Houston Mavericks": HOUSTON_ROCKETS,
    "New Orleans Buccaneers": NEW_ORLEANS_PELICANS,
    "Minnesota Muskies": MINNESOTA_TIMBERWOLVES,
    "New Jersey Americans": NEW_JERSEY_AMERICANS,
    "Pittsburgh Pipers": PITTSBURGH_IRONMEN,
    "New York Nets": BROOKLYN_NETS,
    "Miami Floridians": MIAMI_HEAT,
    "Minnesota Pipers": MINNESOTA_TIMBERWOLVES,
    "Los Angeles Stars": LOS_ANGELES_LAKERS,
    "Carolina Cougars": CAROLINA_COUGARS,
    "Utah Stars": UTAH_JAZZ,
    "Buffalo Braves": BUFFALO_BRAVES,
    "Pittsburgh Condors": PITTSBURGH_IRONMEN,
    "The Floridians": MIAMI_HEAT,
    "Virginia Squires": VIRGINIA_SQUIRES,
    "Texas Chaparrals": DALLAS_MAVERICKS,
    "Memphis Pros": MEMPHIS_GRIZZLIES,
    "Kansas City-Omaha Kings": KANSAS_CITY_KINGS,
    "Memphis Tams": MEMPHIS_GRIZZLIES,
    "San Diego Conquistadors": SAN_DIEGO_ROCKETS,
    "Capital Bullets": CAPITAL_BULLETS,
    "New Orleans Jazz": NEW_ORLEANS_PELICANS,
    "Spirits of St. Louis": ST_LOUIS_BOMBERS,
    "Memphis Sounds": MEMPHIS_GRIZZLIES,
    "Washington Bullets": WASHINGTON_WIZARDS,
    "Kansas City Kings": KANSAS_CITY_KINGS,
    "San Diego Sails": SAN_DIEGO_ROCKETS,
    "New Jersey Nets": BROOKLYN_NETS,
    "San Diego Clippers": LA_CLIPPERS,
    "Vancouver Grizzlies": VANCOUVER_GRIZZLIES,
    "New Orleans Hornets": NEW_ORLEANS_PELICANS,
    "Charlotte Bobcats": CHARLOTTE_HORNETS,
    "New Orleans/Oklahoma City Hornets": NEW_ORLEANS_PELICANS,
}
WELLS_FARGO_CENTER = "Wells Fargo Center"
PAYCOM_CENTER = "Paycom Center"
TOYOTA_CENTER = "Toyota Center"
UNITED_CENTER = "United Center"
AMERICAN_AIRLINES_CENTER = "American Airlines Center"
KIA_CENTER = "Kia Center"
MADISON_SQUARE_GARDEN = "Madison Square Garden"
FISERV_FORUM = "Fiserv Forum"
BALL_ARENA = "Ball Arena"
GAINBRIDGE_FIELDHOUSE = "Gainbridge Fieldhouse"
LITTLE_CAESARS_ARENA = "Little Caesars Arena"
SPECTRUM_CENTER = "Spectrum Center"
KASEYA_CENTER = "Kaseya Center"
FEDEX_FORUM = "FedExForum"
TD_GARDEN = "TD Garden"
MODA_CENTER = "Moda Center"
CRYPTO_DOT_COM_ARENA = "Crypto.com Arena"
SCOTIABANK_ARENA = "Scotiabank Arena"
SMOOTHIE_KING_CENTER = "Smoothie King Center"
STATE_FARM_ARENA = "State Farm Arena"
ROCKET_MORTGAGE_FIELDHOUSE = "Rocket Mortgage FieldHouse"
DELTA_CENTER = "Delta Center"
BARCLAYS_CENTER = "Barclays Center"
GOLDEN_1_CENTER = "Golden 1 Center"
CAPITAL_ONE_ARENA = "Capital One Arena"
TARGET_CENTER = "Target Center"
FOOTPRINT_CENTER = "Footprint Center"
CHASE_CENTER = "Chase Center"
FROST_BANK_CENTER = "Frost Bank Center"
MOODY_CENTER = "Moody Center"
ARCO_ARENA = "ARCO Arena"
BMO_HARRIS_BRADLEY_CENTER = "BMO Harris Bradley Center"
ORACLE_ARENA = "Oracle Arena"
AMALIE_ARENA = "Amalie Arena"
ESPN_WIDE_WORLD_OF_SPORTS = "The Arena (WWOS)"
STATE_FARM_FIELDHOUSE = "State Farm Field House"
VISA_ATHLETIC_CENTER = "Visa Athletic Center"
ROBERT_F_KENNEDY_MEMORIAL_STADIUM = "Robert F. Kennedy Memorial Stadium"
ORIOLE_PARK = "Oriole Park at Camden Yards"
TOYOTA_STADIUM = "Toyota Stadium"
ANGEL_STADIUM = "Angel Stadium of Anaheim"
PANJIN_JINXIU_STADIUM = "Panjin Jinxiu Stadium"
THE_PALACE_OF_AUBURN_HILLS = "The Palace of Auburn Hills"
ADVENTHEALTH_ARENA = "AdventHealth Arena"
PAYPAL_PARK = "PayPal Park"
INTUIT_DOME = "Intuit Dome"
ACCOR_ARENA = "Accor Arena"
SIXTYNINTH_REGIMENT_ARMORY = "69th Regiment Armory, New York, New York"
ONONDAGA_WAR_MEMORIAL = "Onondaga War Memorial, Syracuse, New York"
KIEL_AUDITORIUM = "Kiel Auditorium, St. Louis, Missouri"
MIAMI_BEACH_AUDITORIUM = "Miami Beach Auditorium, Miami Beach, Florida"
DETROIT_OLYMPIA = "Detroit Olympia, Detroit, Michigan"
CONVENTION_HALL = "Convention Hall, Philadelphia, Pennsylvania"
BALTIMORE_CIVIC_CENTER = "Baltimore Civic Center, Baltimore, Maryland"
INDIANA_STATE_FAIR_COLISEUM = "Indiana State Fair Coliseum, Indianapolis, Indiana"
LOUISVILLE_CONVENTION_CENTER = "Louisville Convention Center, Louisville, Kentucky"
LOYOLA_FIELD_HOUSE = "Loyola Field House, New Orleans, Louisiana"
LONG_ISLAND_ARENA = "Long Island Arena, Commack, New York"
METROPOLITAN_SPORTS_CENTER = "Metropolitan Sports Center, Bloomington, Minnesota"
SAM_HOUSTON_COLISEUM = "Sam Houston Coliseum, Houston, Texas"
OAKLAND_ALAMEDA_COUNTY_COLISEUM_ARENA = (
    "Oakland-Alameda County Coliseum Arena, Oakland, California"
)
DULUTH_ARENA_AUDITORIUM = "Duluth Arena Auditorium, Duluth, Minnesota"
LOS_ANGELES_MEMORIAL_SPORTS_ARENA = (
    "Los Angeles Memorial Sports Arena, Los Angeles, California"
)
DENVER_AUDITORIUM_ARENA = "Denver Auditorium Arena, Denver, Colorado"
MIAMI_BEACH_CONVENTION_HALL = "Miami Beach Convention Hall, Miami Beach, Florida"
TARRANT_COUNTY_CONVENTION_CENTER = "Tarrant County Convention Center, Ft. Worth, Texas"
WEST_PALM_BEACH_AUDITORIUM = "West Palm Beach Auditorium, West Palm Beach, Florida"
BLACKHAM_COLISEUM = "Blackham Coliseum, Lafeyette, Louisiana"
ST_LOUIS_ARENA = "St. Louis Arena, St. Louis, Missouri"
MIAMI_DADE_JUNIOR_COLLEGE_NORTH = "Miami-Dade Junior College North, Miami, Florida"
SAN_JOSE_CIVIC_AUDITORIUM = "San Jose Civic Auditorium, San Jose, California"
ST_PAUL_AUDITORIUM = "St. Paul Auditorium, St. Paul, Minnesota"
ANDERSON_HIGHSCHOOL_WIGWAM = "Anderson High School Wigwam, Anderson, Indiana"
ISLAND_GARDEN = "Island Garden, West Hempstead, New York"
ULINE_ARENA = "Uline Arena, Washington, District of Columbia"
GREENSBORO_COLISEUM = "Greensboro Coliseum, Greensboro, North Carolina"
CHARLOTTE_COLISEUM = "Charlotte Coliseum, Charlotte, North Carolina"
DINNER_KEY_AUDITORIUM = "Dinner Key Auditorium, Miami, Florida"
JS_DORTON_ARENA = "J.S. Dorton Arena, Raleigh, North Carolina"
MIDSOUTH_COLISEUM = "Mid-South Coliseum, Memphis, Tennessee"
COMMUNITY_ARENA = "Community Arena, Steubenville, Ohio"
PITTSBURGH_CIVIC_ARENA = "Pittsburgh Civic Arena, Pittsburgh, Pennsylvania"
SELLAND_ARENA = "Selland Arena, Fresno, California"
LEVITT_ARENA = "Levitt Arena, Wichita, Kansas"
REID_GYMNASIUM = "Reid Gymnasium, Cullowhee, North Carolina"
JACKSONVILLE_COLISEUM = "Jacksonville Coliseum, Jacksonville, Florida"
ANAHEIM_CONVENTION_CENTER = "Anaheim Convention Center, Anaheim, California"
LONG_BEACH_ARENA = "Long Beach Arena, Long Beach, California"
SALT_PALACE = "Salt Palace, Salt Lake City, Utah"
NORFOLK_SCOPE = "Norfolk Scope, Norfolk, Virginia"
FREEDOM_HALL = "Freedom Hall, Louisville, Kentucky"
REYNOLDS_COLISEUM = "Reynolds Coliseum, Raleigh, North Carolina"
HAMPTON_COLISEUM = "Hampton Coliseum, Hampton, Virginia"
CURTIS_HIXON_HALL = "Curtis Hixon Hall, Tampa, Florida"
SIKESTON_FIELD_HOUSE = "Sikeston Field House, Sikeston, Missouri"
SWENSON_GYM = "Swenson Gym, Ogden, Utah"
MOBY_GYM = "Moby Gym, Ft. Collins, Colorado"
LUBBOCK_MUNICIPAL_COLISEUM = "Lubbock Municipal Coliseum, Lubbock, Texas"
RICHMOND_ARENA = "Richmond Arena, Richmond, Virginia"
UTAH_STATE_SPECTRUM = "Utah State Spectrum, Logan, Utah"
ROANOKE_CIVIC_CENTER = "Roanoke Civic Center, Roanoke, Virginia"
SALEM_CIVIC_CENTER = "Salem Civic Center, Salem, Virginia"
HOFSTRA_PHYSICAL_FITNESS_CENTER = "Hofstra Physical Fitness Center, Hempstead, New York"
BAYFRONT_ARENA = "Bayfront Arena, St. Petersburg, Florida"
WINSTON_SALEM_MEMORIAL_COLISEUM = (
    "Winston-Salem Memorial Coliseum, Winston-Salem, North Carolina"
)
RICHMOND_COLISEUM = "Richmond Coliseum, Richmond, Virginia"
MAPLE_LEAF_GARDENS = "Maple Leaf Gardens, Toronto, Canada"
UNIONTOWN_HIGHSCHOOL = "Uniontown High School, Uniontown, Pennsylvania"
NASSAU_VETERANS_MEMORIAL_COLISEUM = (
    "Nassau Veterans Memorial Coliseum, Uniondale, New York"
)
TUCSON_COMMUNITY_CENTER = "Tucson Community Center, Tucson, Arizona"
FAIR_PARK = "Fair Park, Birmingham, Alabama"
INDIANA_ASSEMBLY_HALL = "Indiana Assembly Hall, Bloomington, Indiana"
WAR_MEMORIAL_COLISEUM = "War Memorial Coliseum, Fort Wayne, Indiana"
PETERSON_GYM = "Peterson Gym, San Diego, California"
MABEE_CENTER = "Mabee Center, Tulsa, Oklahoma"
ST_PAUL_CIVIC_CENTER = "St. Paul Civic Center, St. Paul, Minnesota"
HEMISFAIR_ARENA = "HemisFair Arena, San Antonio, Texas"
GOLDEN_HALL = "Golden Hall, San Diego, California"
MEMORIAL_COLISEUM = "Memorial Coliseum, Lexington, Kentucky"
CINCINNATI_RIVER_COLISEUM = "Cincinnati Riverfront Coliseum, Cincinnati, Ohio"
EA_DIDDLE_ARENA = "E.A. Diddle Arena, Bowling Green, Kentucky"
HULMAN_CIVIC_UNIVERSITY_CENTER = "Hulman Civic-University Center, Terre Haute, Indiana"
MARKET_SQUARE_ARENA = "Market Square Arena, Indianapolis, Indiana"
SAN_DIEGO_SPORTS_ARENA = "San Diego Sports Arena, San Diego, California"
MCNICOLS_SPORTS_ARENA = "McNichols Sports Arena, Denver, Colorado"
GENERAL_MOTORS_PLACE = "General Motors Place, Vancouver, Canada"
KEYARENA = "KeyArena at Seattle Center, Seattle, Washington"
STAPLES_CENTER = "STAPLES Center, Los Angeles, California"
TD_WATERHOUSE_CENTER = "TD Waterhouse Centre, Orlando, Florida"
REUNION_ARENA = "Reunion Arena, Dallas, Texas"
COMPAQ_CENTER = "Compaq Center, Houston, Texas"
ALAMODOME = "Alamodome, San Antonio, Texas"
CONTINENTAL_AIRLINES_ARENA = "Continental Airlines Arena, East Rutherford, New Jersey"
FLEETCENTER = "FleetCenter, Boston, Massachusetts"
ROSE_GARDEN_ARENA = "Rose Garden Arena, Portland, Oregon"
PEPSI_CENTER = "Pepsi Center, Denver, Colorado"
THE_ARENA = "The Arena in Oakland, Oakland, California"
AMERICA_WEST_ARENA = "America West Arena, Phoenix, Arizona"
FIRST_UNION_CENTER = "First Union Center, Philadelphia, Pennsylvania"
AIR_CANADA_CENTER = "Air Canada Centre, Toronto, Canada"
PHILIPS_ARENA = "Philips Arena, Atlanta, Georgia"
MCI_CENTER = "MCI Center, Washington, District of Columbia"
BRADLEY_CENTER = "Bradley Center, Milwaukee, Wisconsin"
GUND_ARENA = "Gund Arena, Cleveland, Ohio"
CONSECO_FIELDHOUSE = "Conseco Fieldhouse, Indianapolis, Indiana"
PYRAMID_ARENA = "Pyramid Arena, Memphis, Tennessee"
AMERICAN_AIRLINES_ARENA = "AmericanAirlines Arena, Miami, Florida"
NEW_ORLEANS_ARENA = "New Orleans Arena, New Orleans, Louisiana"
SBC_CENTER = "SBC Center, San Antonio, Texas"
WACHOVIA_CENTER = "Wachovia Center, Philadelphia, Pennsylvania"
FORD_CENTER = "Ford Center, Oklahoma City, Oklahoma"
CHARLOTTE_BOBCATS_ARENA = "Charlotte Bobcats Arena, Charlotte, North Carolina"
AMWAY_ARENA = "Amway Arena, Orlando, Florida"
ENERGYSOLUTIONS_ARENA = "EnergySolutions Arena, Salt Lake City, Utah"
QUICKEN_LOANS_ARENA = "Quicken Loans Arena, Cleveland, Ohio"
US_AIRWAYS_CENTER = "US Airways Center, Phoenix, Arizona"
VERIZON_CENTER = "Verizon Center, Washington, District of Columbia"
ATT_CENTER = "AT&T Center, San Antonio, Texas"
IZOD_CENTER = "Izod Center, East Rutherford, New Jersey"
CHESAPEAKE_ENERGY_ARENA = "Chesapeake Energy Arena, Oklahoma City, Oklahoma"
SLEEP_TRAIN_ARENA = "Sleep Train Arena, Sacramento, California"
BANKERS_LIFE_FIELDHOUSE = "Bankers Life Fieldhouse, Indianapolis, Indiana"
TIME_WARNER_CABLE_ARENA = "Time Warner Cable Arena, Charlotte, North Carolina"
VIVINT_SMART_HOME_ARENA = "Vivint Smart Home Arena, Salt Lake City, Utah"
O2_ARENA = "The O2 Arena, London, United Kingdom"
TALKING_STICK_RESORT_ARENA = "Talking Stick Resort Arena, Phoenix, Arizona"
MEXICO_CITY_ARENA = "Mexico City Arena, Mexico City, Mexico"
UW_MILWAUKEE_PANTHER_ARENA = "UW–Milwaukee Panther Arena, Milwaukee, Wisconsin"
HP_FIELD_HOUSE = "HP Field House, Bay Lake, Florida"
PHOENIX_SUNS_ARENA = "Phoenix Suns Arena, Phoenix, Arizona"
FTX_ARENA = "FTX Arena, Miami, Florida"
T_MOBILE_ARENA = "T-Mobile Arena, Las Vegas, Nevada"
ROCKET_ARENA = "Rocket Arena, Cleveland, Ohio"
BOK_CENTER = "24787"
CLIMATE_PLEDGE_ARENA = "27140"
STAN_SHERIFF_CENTER = "29758"
GREAT_WESTERN_FORUM = "26965"
MIAMI_ARENA = "28215"
CHICAGO_STADIUM = "26964"
YOKOHAMA_ARENA = "16381"
OMNI_COLISEUM = "28151"
ORLANDO_ARENA = "28218"
THE_FORUM = "20173"
THOMAS_AND_MACK_CENTER = "24731"
PONTIAC_SILVERDOME = "18379"
KEMPER_ARENA = "24670"
TACOMA_DOME = "20117"
JOE_LOUIS_ARENA = "24619"
MUNICIPAL_AUDITORIUM = "21237"
LOUISIANA_SUPERDOME = "28181"
COW_PALACE = "24673"
INTERNATIONAL_AMPITHEATRE = "19769"
TOKYO_METROPOLITAN_GYMNASIUM = "28141"
LLOYD_NOBLE_CENTER = "LLoyd Noble Center, Norman, Oklahoma"
PETE_MARAVICH_ASSEMBLY_CENTER = "Pete Maravich Assembly Center, Baton Rouge, Louisiana"
POWER_BALANCE_PAVILION = "Power Balance Pavilion, Sacramento, California"
PRUDENTIAL_CENTER = "Prudential Center, Newark, New Jersey"
THE_CENTER = "The Center, Philadelphia, Pennsylvania"
ETIHAD_ARENA = "20454"
SAITAMA_SUPER_ARENA = "17494"
ROGERS_PLACE = "27210"
BELL_CENTER = "23587"
NBA_VENUE_IDENTITY_MAP: dict[str, str] = {
    # SportsReference
    "Wells Fargo Center": WELLS_FARGO_CENTER,
    "Paycom Center": PAYCOM_CENTER,
    "Toyota Center": TOYOTA_CENTER,
    "United Center": UNITED_CENTER,
    "American Airlines Center": AMERICAN_AIRLINES_CENTER,
    "Kia Center": KIA_CENTER,
    "Madison Square Garden": MADISON_SQUARE_GARDEN,
    "Fiserv Forum": FISERV_FORUM,
    "Ball Arena": BALL_ARENA,
    "Gainbridge Fieldhouse": GAINBRIDGE_FIELDHOUSE,
    "Little Caesars Arena": LITTLE_CAESARS_ARENA,
    "Spectrum Center": SPECTRUM_CENTER,
    "Kaseya Center": KASEYA_CENTER,
    "FedExForum": FEDEX_FORUM,
    "TD Garden": TD_GARDEN,
    "Moda Center": MODA_CENTER,
    "Crypto.com Arena": CRYPTO_DOT_COM_ARENA,
    "Scotiabank Arena": SCOTIABANK_ARENA,
    "Smoothie King Center": SMOOTHIE_KING_CENTER,
    "State Farm Arena": STATE_FARM_ARENA,
    "Rocket Mortgage FieldHouse": ROCKET_MORTGAGE_FIELDHOUSE,
    "Delta Center": DELTA_CENTER,
    "Barclays Center": BARCLAYS_CENTER,
    "Golden 1 Center": GOLDEN_1_CENTER,
    "Capital One Arena": CAPITAL_ONE_ARENA,
    "Target Center": TARGET_CENTER,
    "Footprint Center": FOOTPRINT_CENTER,
    "Chase Center": CHASE_CENTER,
    "Frost Bank Center": FROST_BANK_CENTER,
    "Moody Center": MOODY_CENTER,
    "ARCO Arena": ARCO_ARENA,
    "BMO Harris Bradley Center": BMO_HARRIS_BRADLEY_CENTER,
    "Oracle Arena": ORACLE_ARENA,
    "Amalie Arena": AMALIE_ARENA,
    "The Arena (WWOS)": ESPN_WIDE_WORLD_OF_SPORTS,
    "State Farm Field House": STATE_FARM_FIELDHOUSE,
    "Visa Athletic Center": VISA_ATHLETIC_CENTER,
    "ESPN Wide World of Sports Complex": ESPN_WIDE_WORLD_OF_SPORTS,
    "Robert F. Kennedy Memorial Stadium": ROBERT_F_KENNEDY_MEMORIAL_STADIUM,
    "Oriole Park at Camden Yards": ORIOLE_PARK,
    "Toyota Stadium": TOYOTA_STADIUM,
    "Angel Stadium of Anaheim": ANGEL_STADIUM,
    "Panjin Jinxiu Stadium": PANJIN_JINXIU_STADIUM,
    "The Palace of Auburn Hills": THE_PALACE_OF_AUBURN_HILLS,
    "AdventHealth Arena": ADVENTHEALTH_ARENA,
    "PayPal Park": PAYPAL_PARK,
    "Intuit Dome": INTUIT_DOME,
    "Accor Arena": ACCOR_ARENA,
    "Madison Square Garden (III), New York, New York": MADISON_SQUARE_GARDEN,
    "Boston Garden, Boston, Massachusetts": TD_GARDEN,
    "69th Regiment Armory, New York, New York": SIXTYNINTH_REGIMENT_ARMORY,
    "Onondaga War Memorial, Syracuse, New York": ONONDAGA_WAR_MEMORIAL,
    "Kiel Auditorium, St. Louis, Missouri": KIEL_AUDITORIUM,
    "Miami Beach Auditorium, Miami Beach, Florida": MIAMI_BEACH_AUDITORIUM,
    "Detroit Olympia, Detroit, Michigan": DETROIT_OLYMPIA,
    "Convention Hall, Philadelphia, Pennsylvania": CONVENTION_HALL,
    "Baltimore Civic Center, Baltimore, Maryland": BALTIMORE_CIVIC_CENTER,
    "Indiana State Fair Coliseum, Indianapolis, Indiana": INDIANA_STATE_FAIR_COLISEUM,
    "Louisville Convention Center, Louisville, Kentucky": LOUISVILLE_CONVENTION_CENTER,
    "Loyola Field House, New Orleans, Louisiana": LOYOLA_FIELD_HOUSE,
    "Long Island Arena, Commack, New York": LONG_ISLAND_ARENA,
    "Metropolitan Sports Center, Bloomington, Minnesota": METROPOLITAN_SPORTS_CENTER,
    "Sam Houston Coliseum, Houston, Texas": SAM_HOUSTON_COLISEUM,
    "Oakland-Alameda County Coliseum Arena, Oakland, California": OAKLAND_ALAMEDA_COUNTY_COLISEUM_ARENA,
    "Duluth Arena Auditorium, Duluth, Minnesota": DULUTH_ARENA_AUDITORIUM,
    "Los Angeles Memorial Sports Arena, Los Angeles, California": LOS_ANGELES_MEMORIAL_SPORTS_ARENA,
    "Denver Auditorium Arena, Denver, Colorado": DENVER_AUDITORIUM_ARENA,
    "Miami Beach Convention Hall, Miami Beach, Florida": MIAMI_BEACH_CONVENTION_HALL,
    "Tarrant County Convention Center, Ft. Worth, Texas": TARRANT_COUNTY_CONVENTION_CENTER,
    "West Palm Beach Auditorium, West Palm Beach, Florida": WEST_PALM_BEACH_AUDITORIUM,
    "Blackham Coliseum, Lafeyette, Louisiana": BLACKHAM_COLISEUM,
    "St. Louis Arena, St. Louis, Missouri": ST_LOUIS_ARENA,
    "Miami-Dade Junior College North, Miami, Florida": MIAMI_DADE_JUNIOR_COLLEGE_NORTH,
    "San Jose Civic Auditorium, San Jose, California": SAN_JOSE_CIVIC_AUDITORIUM,
    "St. Paul Auditorium, St. Paul, Minnesota": ST_PAUL_AUDITORIUM,
    "Anderson High School Wigwam, Anderson, Indiana": ANDERSON_HIGHSCHOOL_WIGWAM,
    "Island Garden, West Hempstead, New York": ISLAND_GARDEN,
    "Uline Arena, Washington, District of Columbia": ULINE_ARENA,
    "Greensboro Coliseum, Greensboro, North Carolina": GREENSBORO_COLISEUM,
    "Charlotte Coliseum, Charlotte, North Carolina": CHARLOTTE_COLISEUM,
    "Dinner Key Auditorium, Miami, Florida": DINNER_KEY_AUDITORIUM,
    "J.S. Dorton Arena, Raleigh, North Carolina": JS_DORTON_ARENA,
    "Mid-South Coliseum, Memphis, Tennessee": MIDSOUTH_COLISEUM,
    "Community Arena, Steubenville, Ohio": COMMUNITY_ARENA,
    "Pittsburgh Civic Arena, Pittsburgh, Pennsylvania": PITTSBURGH_CIVIC_ARENA,
    "Selland Arena, Fresno, California": SELLAND_ARENA,
    "Levitt Arena, Wichita, Kansas": LEVITT_ARENA,
    "Reid Gymnasium, Cullowhee, North Carolina": REID_GYMNASIUM,
    "Jacksonville Coliseum, Jacksonville, Florida": JACKSONVILLE_COLISEUM,
    "Anaheim Convention Center, Anaheim, California": ANAHEIM_CONVENTION_CENTER,
    "Long Beach Arena, Long Beach, California": LONG_BEACH_ARENA,
    "Salt Palace, Salt Lake City, Utah": SALT_PALACE,
    "Norfolk Scope, Norfolk, Virginia": NORFOLK_SCOPE,
    "Freedom Hall, Louisville, Kentucky": FREEDOM_HALL,
    "Reynolds Coliseum, Raleigh, North Carolina": REYNOLDS_COLISEUM,
    "Hampton Coliseum, Hampton, Virginia": HAMPTON_COLISEUM,
    "Curtis Hixon Hall, Tampa, Florida": CURTIS_HIXON_HALL,
    "Sikeston Field House, Sikeston, Missouri": SIKESTON_FIELD_HOUSE,
    "Swenson Gym, Ogden, Utah": SWENSON_GYM,
    "Moby Gym, Ft. Collins, Colorado": MOBY_GYM,
    "Lubbock Municipal Coliseum, Lubbock, Texas": LUBBOCK_MUNICIPAL_COLISEUM,
    "Richmond Arena, Richmond, Virginia": RICHMOND_ARENA,
    "Madison Square Garden (IV), New York, New York": MADISON_SQUARE_GARDEN,
    "Utah State Spectrum, Logan, Utah": UTAH_STATE_SPECTRUM,
    "Roanoke Civic Center, Roanoke, Virginia": ROANOKE_CIVIC_CENTER,
    "Salem Civic Center, Salem, Virginia": SALEM_CIVIC_CENTER,
    "Hofstra Physical Fitness Center, Hempstead, New York": HOFSTRA_PHYSICAL_FITNESS_CENTER,
    "Bayfront Arena, St. Petersburg, Florida": BAYFRONT_ARENA,
    "Winston-Salem Memorial Coliseum, Winston-Salem, North Carolina": WINSTON_SALEM_MEMORIAL_COLISEUM,
    "Richmond Coliseum, Richmond, Virginia": RICHMOND_COLISEUM,
    "Maple Leaf Gardens, Toronto, Canada": MAPLE_LEAF_GARDENS,
    "Uniontown High School, Uniontown, Pennsylvania": UNIONTOWN_HIGHSCHOOL,
    "Nassau Veterans Memorial Coliseum, Uniondale, New York": NASSAU_VETERANS_MEMORIAL_COLISEUM,
    "Tucson Community Center, Tucson, Arizona": TUCSON_COMMUNITY_CENTER,
    "Fair Park, Birmingham, Alabama": FAIR_PARK,
    "Indiana Assembly Hall, Bloomington, Indiana": INDIANA_ASSEMBLY_HALL,
    "War Memorial Coliseum, Fort Wayne, Indiana": WAR_MEMORIAL_COLISEUM,
    "Peterson Gym, San Diego, California": PETERSON_GYM,
    "Mabee Center, Tulsa, Oklahoma": MABEE_CENTER,
    "St. Paul Civic Center, St. Paul, Minnesota": ST_PAUL_CIVIC_CENTER,
    "HemisFair Arena, San Antonio, Texas": HEMISFAIR_ARENA,
    "Golden Hall, San Diego, California": GOLDEN_HALL,
    "Memorial Coliseum, Lexington, Kentucky": MEMORIAL_COLISEUM,
    "Cincinnati Riverfront Coliseum, Cincinnati, Ohio": CINCINNATI_RIVER_COLISEUM,
    "E.A. Diddle Arena, Bowling Green, Kentucky": EA_DIDDLE_ARENA,
    "Hulman Civic-University Center, Terre Haute, Indiana": HULMAN_CIVIC_UNIVERSITY_CENTER,
    "Market Square Arena, Indianapolis, Indiana": MARKET_SQUARE_ARENA,
    "San Diego Sports Arena, San Diego, California": SAN_DIEGO_SPORTS_ARENA,
    "McNichols Sports Arena, Denver, Colorado": MCNICOLS_SPORTS_ARENA,
    "General Motors Place, Vancouver, Canada": GENERAL_MOTORS_PLACE,
    "KeyArena at Seattle Center, Seattle, Washington": KEYARENA,
    "STAPLES Center, Los Angeles, California": STAPLES_CENTER,
    "TD Waterhouse Centre, Orlando, Florida": TD_WATERHOUSE_CENTER,
    "AmericanAirlines Arena, Miami, Florida": AMERICAN_AIRLINES_ARENA,
    "Reunion Arena, Dallas, Texas": REUNION_ARENA,
    "Compaq Center, Houston, Texas": COMPAQ_CENTER,
    "Alamodome, San Antonio, Texas": ALAMODOME,
    "Continental Airlines Arena, East Rutherford, New Jersey": CONTINENTAL_AIRLINES_ARENA,
    "The Palace of Auburn Hills, Auburn Hills, Michigan": THE_PALACE_OF_AUBURN_HILLS,
    "Target Center, Minneapolis, Minnesota": TARGET_CENTER,
    "ARCO Arena (II), Sacramento, California": ARCO_ARENA,
    "FleetCenter, Boston, Massachusetts": FLEETCENTER,
    "Rose Garden Arena, Portland, Oregon": ROSE_GARDEN_ARENA,
    "Pepsi Center, Denver, Colorado": PEPSI_CENTER,
    "Delta Center, Salt Lake City, Utah": DELTA_CENTER,
    "The Arena in Oakland, Oakland, California": THE_ARENA,
    "America West Arena, Phoenix, Arizona": AMERICA_WEST_ARENA,
    "First Union Center, Philadelphia, Pennsylvania": FIRST_UNION_CENTER,
    "Air Canada Centre, Toronto, Canada": AIR_CANADA_CENTER,
    "Philips Arena, Atlanta, Georgia": PHILIPS_ARENA,
    "MCI Center, Washington, District of Columbia": MCI_CENTER,
    "Bradley Center, Milwaukee, Wisconsin": BRADLEY_CENTER,
    "United Center, Chicago, Illinois": UNITED_CENTER,
    "Gund Arena, Cleveland, Ohio": GUND_ARENA,
    "Conseco Fieldhouse, Indianapolis, Indiana": CONSECO_FIELDHOUSE,
    "Pyramid Arena, Memphis, Tennessee": PYRAMID_ARENA,
    "American Airlines Center, Dallas, Texas": AMERICAN_AIRLINES_CENTER,
    "New Orleans Arena, New Orleans, Louisiana": NEW_ORLEANS_ARENA,
    "SBC Center, San Antonio, Texas": SBC_CENTER,
    "Toyota Center, Houston, Texas": TOYOTA_CENTER,
    "Wachovia Center, Philadelphia, Pennsylvania": WACHOVIA_CENTER,
    "FedEx Forum, Memphis, Tennessee": FEDEX_FORUM,
    "Oakland Arena, Oakland, California": OAKLAND_ALAMEDA_COUNTY_COLISEUM_ARENA,
    "Ford Center, Oklahoma City, Oklahoma": FORD_CENTER,
    "TD Banknorth Garden, Boston, Massachusetts": TD_GARDEN,
    "Charlotte Bobcats Arena, Charlotte, North Carolina": CHARLOTTE_BOBCATS_ARENA,
    "Amway Arena, Orlando, Florida": AMWAY_ARENA,
    "Oracle Arena, Oakland, California": ORACLE_ARENA,
    "EnergySolutions Arena, Salt Lake City, Utah": ENERGYSOLUTIONS_ARENA,
    "Quicken Loans Arena, Cleveland, Ohio": QUICKEN_LOANS_ARENA,
    "US Airways Center, Phoenix, Arizona": US_AIRWAYS_CENTER,
    "Verizon Center, Washington, District of Columbia": VERIZON_CENTER,
    "AT&T Center, San Antonio, Texas": ATT_CENTER,
    "Izod Center, East Rutherford, New Jersey": IZOD_CENTER,
    "Chesapeake Energy Arena, Oklahoma City, Oklahoma": CHESAPEAKE_ENERGY_ARENA,
    "Barclays Center, Brooklyn, New York": BARCLAYS_CENTER,
    "Sleep Train Arena, Sacramento, California": SLEEP_TRAIN_ARENA,
    "Bankers Life Fieldhouse, Indianapolis, Indiana": BANKERS_LIFE_FIELDHOUSE,
    "Time Warner Cable Arena, Charlotte, North Carolina": TIME_WARNER_CABLE_ARENA,
    "TD Garden, Boston, Massachusetts": TD_GARDEN,
    "Wells Fargo Center, Philadelphia, Pennsylvania": WELLS_FARGO_CENTER,
    "BMO Harris Bradley Center, Milwaukee, Wisconsin": BMO_HARRIS_BRADLEY_CENTER,
    "Amway Center, Orlando, Florida": AMWAY_ARENA,
    "Moda Center, Portland, Oregon": MODA_CENTER,
    "Smoothie King Center, New Orleans, Louisiana": SMOOTHIE_KING_CENTER,
    "Vivint Smart Home Arena, Salt Lake City, Utah": VIVINT_SMART_HOME_ARENA,
    "The O2 Arena, London, United Kingdom": O2_ARENA,
    "Talking Stick Resort Arena, Phoenix, Arizona": TALKING_STICK_RESORT_ARENA,
    "Mexico City Arena, Mexico City, Mexico": MEXICO_CITY_ARENA,
    "Golden 1 Center, Sacramento, California": GOLDEN_1_CENTER,
    "Spectrum Center, Charlotte, North Carolina": SPECTRUM_CENTER,
    "Capital One Arena, Washington, District of Columbia": CAPITAL_ONE_ARENA,
    "Little Caesars Arena, Detroit, Michigan": LITTLE_CAESARS_ARENA,
    "UW–Milwaukee Panther Arena, Milwaukee, Wisconsin": UW_MILWAUKEE_PANTHER_ARENA,
    "Scotiabank Arena, Toronto, Canada": SCOTIABANK_ARENA,
    "Fiserv Forum, Milwaukee, Wisconsin": FISERV_FORUM,
    "State Farm Arena, Atlanta, Georgia": STATE_FARM_ARENA,
    "Chase Center, San Francisco, California": CHASE_CENTER,
    "AccorHotels Arena, Paris, France": ACCOR_ARENA,
    "Rocket Mortgage Fieldhouse, Cleveland, Ohio": ROCKET_MORTGAGE_FIELDHOUSE,
    "HP Field House, Bay Lake, Florida": HP_FIELD_HOUSE,
    "The Arena, Bay Lake, Florida": THE_ARENA,
    "Visa Athletic Center, Bay Lake, Florida": VISA_ATHLETIC_CENTER,
    "Amalie Arena, Tampa, Florida": AMALIE_ARENA,
    "Phoenix Suns Arena, Phoenix, Arizona": PHOENIX_SUNS_ARENA,
    "Ball Arena, Denver, Colorado": BALL_ARENA,
    "Crypto.com Arena, Los Angeles, California": CRYPTO_DOT_COM_ARENA,
    "FTX Arena, Miami, Florida": FTX_ARENA,
    "Gainbridge Fieldhouse, Indianapolis, Indiana": GAINBRIDGE_FIELDHOUSE,
    "Paycom Center, Oklahoma City, Oklahoma": PAYCOM_CENTER,
    "Vivint Arena, Salt Lake City, Utah": VIVINT_SMART_HOME_ARENA,
    "Footprint Center, Phoenix, Arizona": FOOTPRINT_CENTER,
    "Kaseya Center, Miami, Florida": KASEYA_CENTER,
    "Frost Bank Center, San Antonio, Texas": FROST_BANK_CENTER,
    "Kia Center, Orlando, Florida": KIA_CENTER,
    "T-Mobile Arena, Las Vegas, Nevada": T_MOBILE_ARENA,
    "Moody Center, Austin, Texas": MOODY_CENTER,
    "Intuit Dome, Inglewood, California": INTUIT_DOME,
    "Rocket Arena, Cleveland, Ohio": ROCKET_ARENA,
    "LLoyd Noble Center, Norman, Oklahoma": LLOYD_NOBLE_CENTER,
    "Pete Maravich Assembly Center, Baton Rouge, Louisiana": PETE_MARAVICH_ASSEMBLY_CENTER,
    "Oklahoma City Arena, Oklahoma City, Oklahoma": OAKLAND_ALAMEDA_COUNTY_COLISEUM_ARENA,
    "Power Balance Pavilion, Sacramento, California": POWER_BALANCE_PAVILION,
    "Prudential Center, Newark, New Jersey": PRUDENTIAL_CENTER,
    "The Center, Philadelphia, Pennsylvania": THE_CENTER,
    # SportsDB
    "25994": CRYPTO_DOT_COM_ARENA,
    "29509": DELTA_CENTER,
    "24837": FISERV_FORUM,
    "19870": TOYOTA_CENTER,
    "24836": GOLDEN_1_CENTER,
    "24848": BALL_ARENA,
    "24818": MODA_CENTER,
    "30970": INTUIT_DOME,
    "24819": SMOOTHIE_KING_CENTER,
    "24787": BOK_CENTER,
    "24852": PAYCOM_CENTER,
    "29950": FROST_BANK_CENTER,
    "19860": AMERICAN_AIRLINES_CENTER,
    "24835": LITTLE_CAESARS_ARENA,
    "19333": WELLS_FARGO_CENTER,
    "24830": SCOTIABANK_ARENA,
    "24786": TD_GARDEN,
    "23207": TARGET_CENTER,
    "24847": GAINBRIDGE_FIELDHOUSE,
    "16006": MADISON_SQUARE_GARDEN,
    "22906": ROCKET_MORTGAGE_FIELDHOUSE,
    "23213": FOOTPRINT_CENTER,
    "27140": CLIMATE_PLEDGE_ARENA,
    "29758": STAN_SHERIFF_CENTER,
    "28400": GENERAL_MOTORS_PLACE,
    "17606": ESPN_WIDE_WORLD_OF_SPORTS,
    "24699": AMERICAN_AIRLINES_CENTER,
    "24766": AMWAY_ARENA,
    "19838": STAPLES_CENTER,
    "24762": BALL_ARENA,
    "15830": QUICKEN_LOANS_ARENA,
    "24743": TD_GARDEN,
    "24724": PHILIPS_ARENA,
    "24704": WACHOVIA_CENTER,
    "22727": UNITED_CENTER,
    "19852": ROSE_GARDEN_ARENA,
    "16149": ATT_CENTER,
    "26965": GREAT_WESTERN_FORUM,
    "24614": REUNION_ARENA,
    "15554": MCNICOLS_SPORTS_ARENA,
    "28215": MIAMI_ARENA,
    "24640": AMERICA_WEST_ARENA,
    "24644": MARKET_SQUARE_ARENA,
    "24617": BRADLEY_CENTER,
    "26964": CHICAGO_STADIUM,
    "16381": YOKOHAMA_ARENA,
    "24656": CHARLOTTE_COLISEUM,
    "24620": ARCO_ARENA,
    "28151": OMNI_COLISEUM,
    "28198": SPECTRUM_CENTER,
    "30267": TD_GARDEN,
    "26966": THE_PALACE_OF_AUBURN_HILLS,
    "28218": ORLANDO_ARENA,
    "30268": LOS_ANGELES_MEMORIAL_SPORTS_ARENA,
    "25255": MEMORIAL_COLISEUM,
    "20173": THE_FORUM,
    "24731": THOMAS_AND_MACK_CENTER,
    "18379": PONTIAC_SILVERDOME,
    "24670": KEMPER_ARENA,
    "20117": TACOMA_DOME,
    "24619": JOE_LOUIS_ARENA,
    "24625": SAN_DIEGO_SPORTS_ARENA,
    "28217": UW_MILWAUKEE_PANTHER_ARENA,
    "21237": MUNICIPAL_AUDITORIUM,
    "28181": LOUISIANA_SUPERDOME,
    "24673": COW_PALACE,
    "30266": SPECTRUM_CENTER,
    "19769": INTERNATIONAL_AMPITHEATRE,
    "20105": SAN_JOSE_CIVIC_AUDITORIUM,
    "28141": TOKYO_METROPOLITAN_GYMNASIUM,
    "25267": ANAHEIM_CONVENTION_CENTER,
    "24759": NEW_ORLEANS_ARENA,
    "19862": CHESAPEAKE_ENERGY_ARENA,
    "24851": FTX_ARENA,
    "24761": OAKLAND_ALAMEDA_COUNTY_COLISEUM_ARENA,
    "20454": ETIHAD_ARENA,
    "17494": SAITAMA_SUPER_ARENA,
    "27210": ROGERS_PLACE,
    "23587": BELL_CENTER,
    "21308": STATE_FARM_ARENA,
    "18422": KIA_CENTER,
    "19331": CAPITAL_ONE_ARENA,
    "24838": SPECTRUM_CENTER,
    "28619": KASEYA_CENTER,
    "24727": FEDEX_FORUM,
    "24828": CHASE_CENTER,
    # NBACom
    "Paycom Center, Oklahoma City, OK": PAYCOM_CENTER,
    "Madison Square Garden, New York, NY": MADISON_SQUARE_GARDEN,
    "Target Center, Minneapolis, MN": TARGET_CENTER,
}
NBA_PLAYER_IDENTITY_MAP: dict[str, str] = {}


class NBACombinedLeagueModel(CombinedLeagueModel):
    """NBA combined implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, league_filter: str | None
    ) -> None:
        super().__init__(
            session,
            League.NBA,
            [
                NBANBALeagueModel(session, position=0),
                NBAESPNLeagueModel(session, position=1),
                NBASportsDBLeagueModel(session, position=2),
                NBASportsReferenceLeagueModel(session, position=3),
                NBAOddsPortalLeagueModel(session, position=4),
                NBANBAComLeagueModel(session, position=5),
            ],
            league_filter,
        )

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        return NBA_TEAM_IDENTITY_MAP

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        return NBA_VENUE_IDENTITY_MAP

    @classmethod
    def player_identity_map(cls) -> dict[str, str]:
        return NBA_PLAYER_IDENTITY_MAP

    @classmethod
    def name(cls) -> str:
        return "nba-combined-league-model"



"""Combined player model."""

# pylint: disable=too-many-locals,too-many-branches,too-many-statements,duplicate-code
from ..player_model import PlayerModel
from .null_check import is_null


def create_combined_player_model(
    player_models: list[PlayerModel], identifier: str
) -> PlayerModel:
    """Create a player model by combining many player models."""
    jersey = None
    kicks = None
    fumbles = None
    fumbles_lost = None
    field_goals = None
    field_goals_attempted = None
    offensive_rebounds = None
    assists = None
    turnovers = None
    name = None
    marks = None
    handballs = None
    disposals = None
    goals = None
    behinds = None
    hit_outs = None
    tackles = None
    rebounds = None
    insides = None
    clearances = None
    clangers = None
    free_kicks_for = None
    free_kicks_against = None
    brownlow_votes = None
    contested_possessions = None
    uncontested_possessions = None
    contested_marks = None
    marks_inside = None
    one_percenters = None
    bounces = None
    goal_assists = None
    percentage_played = None
    birth_date = None
    species = None
    handicap_weight = None
    father = None
    sex = None
    age = None
    starting_position = None
    weight = None
    birth_address = None
    owner = None
    for player_model in player_models:
        player_model_jersey = player_model.jersey
        if not is_null(player_model_jersey):
            jersey = player_model_jersey
        player_model_kicks = player_model.kicks
        if not is_null(player_model_kicks):
            kicks = player_model_kicks
        player_model_fumbles = player_model.fumbles
        if not is_null(player_model_fumbles):
            fumbles = player_model_fumbles
        player_model_fumbles_lost = player_model.fumbles_lost
        if not is_null(player_model_fumbles_lost):
            fumbles_lost = player_model_fumbles_lost
        player_model_field_goals = player_model.field_goals
        if not is_null(player_model_field_goals):
            field_goals = player_model_field_goals
        player_model_field_goals_attempted = player_model.field_goals_attempted
        if not is_null(player_model_field_goals_attempted):
            field_goals_attempted = player_model_field_goals_attempted
        player_model_offensive_rebounds = player_model.offensive_rebounds
        if not is_null(player_model_offensive_rebounds):
            offensive_rebounds = player_model_offensive_rebounds
        player_model_assists = player_model.assists
        if not is_null(player_model_assists):
            assists = player_model_assists
        player_model_turnovers = player_model.turnovers
        if not is_null(player_model_turnovers):
            turnovers = player_model_turnovers
        player_model_name = player_model.name
        if not is_null(player_model_name):
            name = player_model_name
        player_model_marks = player_model.marks
        if not is_null(player_model_marks):
            marks = player_model_marks
        player_model_handballs = player_model.handballs
        if not is_null(player_model_handballs):
            handballs = player_model_handballs
        player_model_disposals = player_model.disposals
        if not is_null(player_model_disposals):
            disposals = player_model_disposals
        player_model_goals = player_model.goals
        if not is_null(player_model_goals):
            goals = player_model_goals
        player_model_behinds = player_model.behinds
        if not is_null(player_model_behinds):
            behinds = player_model_behinds
        player_model_hit_outs = player_model.hit_outs
        if not is_null(player_model_hit_outs):
            hit_outs = player_model_hit_outs
        player_model_tackles = player_model.tackles
        if not is_null(player_model_tackles):
            tackles = player_model_tackles
        player_model_rebounds = player_model.rebounds
        if not is_null(player_model_rebounds):
            rebounds = player_model_rebounds
        player_model_insides = player_model.insides
        if not is_null(player_model_insides):
            insides = player_model_insides
        player_model_clearances = player_model.clearances
        if not is_null(player_model_clearances):
            clearances = player_model_clearances
        player_model_clangers = player_model.clangers
        if not is_null(player_model_clangers):
            clangers = player_model_clangers
        player_model_free_kicks_for = player_model.free_kicks_for
        if not is_null(player_model_free_kicks_for):
            free_kicks_for = player_model_free_kicks_for
        player_model_free_kicks_against = player_model.free_kicks_against
        if not is_null(player_model_free_kicks_against):
            free_kicks_against = player_model_free_kicks_against
        player_model_brownlow_votes = player_model.brownlow_votes
        if not is_null(player_model_brownlow_votes):
            brownlow_votes = player_model_brownlow_votes
        player_model_contested_possessions = player_model.contested_possessions
        if not is_null(player_model_contested_possessions):
            contested_possessions = player_model_contested_possessions
        player_model_uncontested_possessions = player_model.uncontested_possessions
        if not is_null(player_model_uncontested_possessions):
            uncontested_possessions = player_model_uncontested_possessions
        player_model_contested_marks = player_model.contested_marks
        if not is_null(player_model_contested_marks):
            contested_marks = player_model_contested_marks
        player_model_marks_inside = player_model.marks_inside
        if not is_null(player_model_marks_inside):
            marks_inside = player_model_marks_inside
        player_model_one_percenters = player_model.one_percenters
        if not is_null(player_model_one_percenters):
            one_percenters = player_model_one_percenters
        player_model_bounces = player_model.bounces
        if not is_null(player_model_bounces):
            bounces = player_model_bounces
        player_model_goal_assists = player_model.goal_assists
        if not is_null(player_model_goal_assists):
            goal_assists = player_model_goal_assists
        player_model_percentage_played = player_model.percentage_played
        if not is_null(player_model_percentage_played):
            percentage_played = player_model_percentage_played
        player_model_birth_date = player_model.birth_date
        if not is_null(player_model_birth_date):
            birth_date = player_model_birth_date
        player_model_species = player_model.species
        if not is_null(player_model_species):
            species = player_model_species
        player_model_handicap_weight = player_model.handicap_weight
        if not is_null(player_model_handicap_weight):
            handicap_weight = player_model_handicap_weight
        player_model_father = player_model.father
        if not is_null(player_model_father):
            father = player_model_father
        player_model_sex = player_model.sex
        if not is_null(player_model_sex):
            sex = player_model_sex
        player_model_age = player_model.age
        if not is_null(player_model_age):
            age = player_model_age
        player_model_starting_position = player_model.starting_position
        if not is_null(player_model_starting_position):
            starting_position = player_model_starting_position
        player_model_weight = player_model.weight
        if not is_null(player_model_weight):
            weight = player_model_weight
        player_model_birth_address = player_model.birth_address
        if not is_null(player_model_birth_address):
            birth_address = player_model_birth_address
        player_model_owner = player_model.owner
        if not is_null(player_model_owner):
            owner = player_model_owner
    if name is None:
        raise ValueError("name is null")
    if species is None:
        raise ValueError("species is null")
    return PlayerModel(
        identifier=identifier,
        jersey=jersey,
        kicks=kicks,
        fumbles=fumbles,
        fumbles_lost=fumbles_lost,
        field_goals=field_goals,
        field_goals_attempted=field_goals_attempted,
        offensive_rebounds=offensive_rebounds,
        assists=assists,
        turnovers=turnovers,
        name=name,
        marks=marks,
        handballs=handballs,
        disposals=disposals,
        goals=goals,
        behinds=behinds,
        hit_outs=hit_outs,
        tackles=tackles,
        rebounds=rebounds,
        insides=insides,
        clearances=clearances,
        clangers=clangers,
        free_kicks_for=free_kicks_for,
        free_kicks_against=free_kicks_against,
        brownlow_votes=brownlow_votes,
        contested_possessions=contested_possessions,
        uncontested_possessions=uncontested_possessions,
        contested_marks=contested_marks,
        marks_inside=marks_inside,
        one_percenters=one_percenters,
        bounces=bounces,
        goal_assists=goal_assists,
        percentage_played=percentage_played,
        birth_date=birth_date,
        species=species,
        handicap_weight=handicap_weight,
        father=father,
        sex=sex,
        age=age,
        starting_position=starting_position,
        weight=weight,
        birth_address=birth_address,
        owner=owner,
    )



"""Openmeteo weather model."""

import datetime

import openmeteo_requests  # type: ignore
import pandas as pd
import pytest_is_running
import pytz
import requests
import requests_cache
from openmeteo_requests.Client import OpenMeteoRequestsError  # type: ignore
from openmeteo_requests.Client import WeatherApiResponse

from ....cache import MEMORY
from ...weather_model import WeatherModel


def _parse_openmeteo(
    responses: list[WeatherApiResponse], tz: str, dt: datetime.datetime
) -> WeatherModel | None:
    # pylint: disable=broad-exception-caught
    if not responses:
        return None
    response = responses[0]
    try:
        hourly = response.Hourly()
    except Exception:
        return None
    if hourly is None:
        raise ValueError("hourly is null.")
    dt_index = pd.date_range(
        start=pd.to_datetime(hourly.Time(), unit="s"),
        end=pd.to_datetime(hourly.TimeEnd(), unit="s"),
        freq=pd.Timedelta(seconds=hourly.Interval()),
        inclusive="left",
        tz=tz,
    )
    row_count = min(
        len(hourly.Variables(0).ValuesAsNumpy()),  # type: ignore
        len(hourly.Variables(1).ValuesAsNumpy()),  # type: ignore
        len(dt_index),
    )
    hourly_df = pd.DataFrame(
        index=dt_index[:row_count],  # type: ignore
        data={
            "temperature_2m": hourly.Variables(0).ValuesAsNumpy()[:row_count],  # type: ignore
            "relative_humidity_2m": hourly.Variables(1).ValuesAsNumpy()[:row_count],  # type: ignore
        },
    )
    dt = dt.replace(tzinfo=None)
    timezone = pytz.timezone(tz)
    dt = timezone.localize(dt)
    hourly_idx = hourly_df.index.get_indexer([dt], method="nearest")[0]
    temperature = hourly_df.iloc[hourly_idx]["temperature_2m"]  # type: ignore
    relative_humidity = hourly_df.iloc[hourly_idx]["relative_humidity_2m"]  # type: ignore
    return WeatherModel(temperature=temperature, relative_humidity=relative_humidity)


def _create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
) -> WeatherModel | None:
    # pylint: disable=broad-exception-caught
    client = openmeteo_requests.Client(session=session)
    try:
        responses = client.weather_api(
            "https://historical-forecast-api.open-meteo.com/v1/forecast",
            params={
                "latitude": latitude,
                "longitude": longitude,
                "start_date": str((dt - datetime.timedelta(days=1.0)).date()),
                "end_date": str(dt.date()),
                "hourly": [
                    "temperature_2m",
                    "relative_humidity_2m",
                    "dew_point_2m",
                    "apparent_temperature",
                    "precipitation",
                    "rain",
                    "snowfall",
                    "snow_depth",
                    "weather_code",
                    "pressure_msl",
                    "surface_pressure",
                    "cloud_cover",
                    "cloud_cover_low",
                    "cloud_cover_mid",
                    "cloud_cover_high",
                    "et0_fao_evapotranspiration",
                    "vapour_pressure_deficit",
                    "wind_speed_10m",
                    "wind_speed_100m",
                    "wind_direction_10m",
                    "wind_direction_100m",
                    "wind_gusts_10m",
                    "soil_temperature_0_to_7cm",
                    "soil_temperature_7_to_28cm",
                    "soil_temperature_28_to_100cm",
                    "soil_temperature_100_to_255cm",
                    "soil_moisture_0_to_7cm",
                    "soil_moisture_7_to_28cm",
                    "soil_moisture_28_to_100cm",
                    "soil_moisture_100_to_255cm",
                ],
                "daily": [
                    "weather_code",
                    "temperature_2m_max",
                    "temperature_2m_min",
                    "temperature_2m_mean",
                    "apparent_temperature_max",
                    "apparent_temperature_min",
                    "apparent_temperature_mean",
                    "sunrise",
                    "sunset",
                    "daylight_duration",
                    "sunshine_duration",
                    "precipitation_sum",
                    "rain_sum",
                    "snowfall_sum",
                    "precipitation_hours",
                    "wind_speed_10m_max",
                    "wind_gusts_10m_max",
                    "wind_direction_10m_dominant",
                    "shortwave_radiation_sum",
                    "et0_fao_evapotranspiration",
                ],
                "timezone": tz,
            },
        )
        return _parse_openmeteo(responses, tz, dt)
    except (
        requests.exceptions.RetryError,
        OpenMeteoRequestsError,
        requests.exceptions.ReadTimeout,
    ):
        return None
    except Exception as e:
        e_text = str(e)
        if "Parameter 'start_date' is out of allowed range from" in e_text:
            return None
        raise e


@MEMORY.cache(ignore=["session"])
def _cached_create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
) -> WeatherModel | None:
    return _create_openmeteo_weather_model(session, latitude, longitude, dt, tz)


def create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
) -> WeatherModel | None:
    """Create a weather model from openmeteo."""
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=3):
        return _cached_create_openmeteo_weather_model(
            session, latitude, longitude, dt, tz
        )
    with session.cache_disabled():
        return _create_openmeteo_weather_model(session, latitude, longitude, dt, tz)



"""The base class for a data model."""

# pylint: disable=too-few-public-methods
import pandas as pd
import requests_cache

from ..proxy_session import create_proxy_session


class Model:
    """The base model class."""

    _session: requests_cache.CachedSession | None

    def __init__(self, session: requests_cache.CachedSession) -> None:
        self._session = session

    @property
    def session(self) -> requests_cache.CachedSession:
        """Fetch a session that can never be null."""
        session = self._session
        if session is None:
            session = create_proxy_session()
            self._session = session
        return session

    def clear_session(self):
        """Clear the session."""
        self._session = None

    def to_frame(self) -> pd.DataFrame:
        """Render the model as a dataframe."""
        raise NotImplementedError("to_frame not implemented by parent class.")



"""Gribstream weather model."""

import datetime
import gzip
import io
import json
import os

import pandas as pd
import pytest_is_running
import pytz
import requests
import requests_cache

from ....cache import MEMORY
from ...weather_model import WeatherModel


def _create_gribstream_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
) -> WeatherModel | None:
    api_key = os.environ.get("GRIBSTREAM_API_KEY")
    if api_key is None:
        return None
    url = "https://gribstream.com/api/v2/nbm/forecasts"
    payload = {
        "forecastedFrom": dt.astimezone(pytz.utc).strftime("%Y-%m-%dT%H:%M:%SZ"),
        "forecastedUntil": dt.astimezone(pytz.utc).strftime("%Y-%m-%dT%H:%M:%SZ"),
        "minHorizon": 12,
        "maxHorizon": 24,
        "coordinates": [{"lat": latitude, "lon": longitude}],
        "variables": [
            {"name": "TMP", "level": "2 m above ground", "info": ""},
            {"name": "DPT", "level": "2 m above ground", "info": ""},
            {"name": "RH", "level": "2 m above ground", "info": ""},
        ],
    }
    try:
        resp = session.post(
            url,
            data=gzip.compress(json.dumps(payload).encode("utf-8")),
            headers={
                "Accept-Encoding": "gzip",
                "Content-Encoding": "gzip",
                "Content-Type": "application/json",
                "Authorization": "Bearer ",
            },
        )
    except requests.exceptions.ConnectionError:
        return None
    if not resp.ok:
        return None
    df = pd.read_csv(io.BytesIO(resp.content), parse_dates=[0, 1])
    idx = df.index.get_indexer([dt], method="nearest")[0]
    temperature = df.iloc[idx]["TMP|2 m above ground|"]  # type: ignore
    relative_humidity = df.iloc[idx]["RH|2 m above ground|"]  # type: ignore
    return WeatherModel(temperature=temperature, relative_humidity=relative_humidity)


@MEMORY.cache(ignore=["session"])
def _cached_create_gribstream_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
) -> WeatherModel | None:
    return _create_gribstream_weather_model(session, latitude, longitude, dt)


def create_gribstream_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
) -> WeatherModel | None:
    """Create a weather model from gribstream."""
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=3):
        return _cached_create_gribstream_weather_model(session, latitude, longitude, dt)
    with session.cache_disabled():
        return _create_gribstream_weather_model(session, latitude, longitude, dt)



"""Tests for the combined game model class."""
import datetime
import os
import unittest

import requests_mock
import requests_cache
from sportsball.data.combined.combined_game_model import create_combined_game_model
from sportsball.data.league import League
from sportsball.data.season_type import SeasonType
from sportsball.data.game_model import GameModel


class TestCombinedGameModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_dt(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        with requests_mock.Mocker() as m:
            game_model = GameModel(
                dt=dt,
                week=None,
                game_number=None,
                venue=None,
                teams=[],
                end_dt=None,
                attendance=None,
                league=str(League.NBA),
                year=None,
                season_type=SeasonType.REGULAR,
                postponed=True,
                play_off=None,
                distance=None,
                dividends=[],
                pot=None,
            )
            names = {}
            coach_names = {}
            combined_game_model = create_combined_game_model(
                game_models=[game_model],
                venue_identity_map={},
                team_identity_map={},
                player_identity_map={},
                session=self._session,
                names=names,
                coach_names=coach_names,
            )
            self.assertEqual(combined_game_model.dt, dt)



"""NBA sports reference league model."""

# pylint: disable=line-too-long

import requests_cache

from ...league import League
from ...sportsreference.sportsreference_league_model import \
    SportsReferenceLeagueModel
from ..position import Position


class NBASportsReferenceLeagueModel(SportsReferenceLeagueModel):
    """NBA Sports Reference implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, position: int | None = None
    ) -> None:
        super().__init__(
            session,
            League.NBA,
            "https://www.basketball-reference.com/boxscores/",
            position=position,
        )

    @classmethod
    def name(cls) -> str:
        return "nba-sportsreference-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        return {str(x): str(x) for x in Position}



"""Tests for the afl AFL league model class."""
import unittest
import os

import requests_cache
from sportsball.data.afl.afl.afl_afl_league_model import _parse_game_info


class TestAFLAFLLeagueModel(unittest.TestCase):

    def setUp(self):
        self.dir = os.path.dirname(__file__)
        self._session = requests_cache.CachedSession(backend="memory")

    """
    def test_parse_game_info(self):
        with open(os.path.join(self.dir, "team-lineups.html")) as handle:
            game_models = list(_parse_game_info(handle.read(), self._session, [
                "Melbourne",
                "Richmond",
                "Collingwood",
                "Essendon",
                "Fremantle",
                "Adelaide Crows",
                "St Kilda",
                "Brisbane Lions",
                "Port Adelaide",
                "North Melbourne",
                "GWS GIANTS",
                "Western Bulldogs",
                "Gold Coast SUNS",
                "Sydney Swans",
                "Carlton",
                "Geelong Cats",
                "Hawthorn",
                "West Coast Eagles",
            ], "https://www.afl.com.au/matches/team-lineups", None))
            first_game = game_models[0]
            self.assertEqual(len(first_game.teams), 2)
            for team in first_game.teams:
                self.assertEqual(len(team.players), 22)
    """



"""HKJC HKJC venue model."""

# pylint: disable=too-many-arguments
import datetime

import requests_cache

from ....cache import MEMORY
from ...google.google_address_model import create_google_address_model
from ...venue_model import VenueModel

SHA_TIN_VENUE_CODE = "ST"
HAPPY_VALLEY_VENUE_CODE = "HV"

ADDRESSES = {
    SHA_TIN_VENUE_CODE: "Sha Tin, China",
    HAPPY_VALLEY_VENUE_CODE: "Hally Valley, China",
}


@MEMORY.cache(ignore=["session"])
def create_hkjc_hkjc_venue_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    venue_code: str,
    race_track: str,
) -> VenueModel:
    """Create a venue model from an HKJC result."""
    address = create_google_address_model(
        ADDRESSES[venue_code],
        session,
        dt,
    )
    name = ", ".join([race_track, venue_code])
    return VenueModel(
        identifier=name,
        name=name,
        address=address,  # pyright: ignore
        is_grass=None,
        is_indoor=None,
        is_turf=race_track.lower() == "turf",
        is_dirt=race_track.lower() == "dirt",
    )



"""AFL ESPN league model."""

# pylint: disable=line-too-long

import requests_cache

from ...espn.espn_league_model import ESPNLeagueModel
from ...league import League

_SEASON_URL = "https://sports.core.api.espn.com/v2/sports/australian-football/leagues/afl/seasons?limit=100"


class AFLESPNLeagueModel(ESPNLeagueModel):
    """AFL ESPN implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, position: int | None = None
    ) -> None:
        super().__init__(_SEASON_URL, League.AFL, session, position=position)

    @classmethod
    def name(cls) -> str:
        return "afl-espn-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        return {}



"""NCAAF ESPN league model."""

# pylint: disable=line-too-long

import requests_cache

from ...espn.espn_league_model import ESPNLeagueModel
from ...league import League
from ...nfl.position import Position

_SEASON_URL = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons?limit=100"


class NCAAFESPNLeagueModel(ESPNLeagueModel):
    """NCAAF ESPN implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, position: int | None = None
    ) -> None:
        super().__init__(_SEASON_URL, League.NCAAF, session, position=position)

    @classmethod
    def name(cls) -> str:
        return "ncaaf-espn-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        return {str(x): str(x) for x in Position}
